# -*- coding: utf-8 -*-
'''

'''
from __future__ import absolute_import, print_function, unicode_literals

# Python
import socket
from salt.ext.six.moves import zip  # pylint: disable=redefined-builtin

# Salt
from salt._compat import ipaddress
from salt.utils.odict import OrderedDict
import salt.utils.dns
from salt.utils.dns import _to_port, _tree, _weighted_order, _data2rec, _data2rec_group
from salt.utils.dns import _lookup_gai, _lookup_dig, _lookup_drill, _lookup_host, _lookup_nslookup
import salt.modules.cmdmod

# Testing
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch


# Debug
import pprint
ppr = pprint.PrettyPrinter(indent=2).pprint


class DNShelpersCase(TestCase):
    '''
    Tests for the parser helpers
    '''
    def test_port(self):
        for right in (1, 42, '123', 65535):
            self.assertEqual(_to_port(right), int(right))

        for wrong in (0, 65536, 100000, 'not-a-port'):
            self.assertRaises(ValueError, _to_port, wrong)

    def test_tree(self):
        test_map = (
            'ex1.nl',
            'o.1.example.eu',
            'a1a.b2b.c3c.example.com'
        )

        res_map = (
            ['ex1.nl'],
            ['o.1.example.eu', '1.example.eu', 'example.eu'],
            ['a1a.b2b.c3c.example.com', 'b2b.c3c.example.com', 'c3c.example.com', 'example.com']
        )

        for domain, result in zip(test_map, res_map):
            self.assertEqual(_tree(domain), result)

    def test_weight(self):
        recs = [
            [],
            [{'weight': 100, 'name': 'nescio'}],
            [
                {'weight': 100, 'name': 'nescio1'},
                {'weight': 100, 'name': 'nescio2'},
                {'weight': 100, 'name': 'nescio3'},
                {'weight': 100, 'name': 'nescio4'},
                {'weight': 100, 'name': 'nescio5'},
                {'weight': 100, 'name': 'nescio6'},
                {'weight': 100, 'name': 'nescio7'},
                {'weight': 100, 'name': 'nescio8'}
            ]
        ]

        # What are the odds of this tripping over a build
        self.assertNotEqual(
            _weighted_order(list(recs[-1])),
            _weighted_order(list(recs[-1]))
        )

        for recset in recs:
            rs_res = _weighted_order(list(recset))
            self.assertTrue(all(rec['name'] in rs_res for rec in recset))

    def test_data2rec(self):
        right = [
            '10.0.0.1',
            '10 mbox.example.com',
            '10 20 30 example.com',
        ]
        schemas = [
            OrderedDict((
                ('address', ipaddress.IPv4Address),
            )),
            OrderedDict((
                ('preference', int),
                ('name', str),
            )),
            OrderedDict((
                ('prio', int),
                ('weight', int),
                ('port', _to_port),
                ('name', str),
            ))
        ]

        results = [
            {'address': ipaddress.IPv4Address(right[0])},
            {'preference': 10, 'name': 'mbox.example.com'},
            {'prio': 10, 'weight': 20, 'port': 30, 'name': 'example.com'}
        ]

        for rdata, rschema, res in zip(right, schemas, results):
            self.assertEqual(_data2rec(rschema, rdata), res)

        wrong = [
            'not-an-ip',
            '10 20 30 toomany.example.com',
            '10 toolittle.example.com',
        ]

        for rdata, rschema in zip(wrong, schemas):
            self.assertRaises(ValueError, _data2rec, rschema, rdata)

    def test_data2group(self):
        right = [
            ['10 mbox.example.com'],
            [
                '10 mbox1.example.com',
                '20 mbox2.example.com',
                '20 mbox3.example.com',
                '30 mbox4.example.com',
                '30 mbox5.example.com',
                '30 mbox6.example.com',
            ],
        ]
        rschema = OrderedDict((
                ('prio', int),
                ('srvr', str),
            ))

        results = [
            OrderedDict([(10, [{'srvr': 'mbox.example.com'}])]),
            OrderedDict([
                (10, [{'srvr': 'mbox1.example.com'}]),
                (20, [{'srvr': 'mbox2.example.com'}, {'srvr': 'mbox3.example.com'}]),
                (30, [{'srvr': 'mbox4.example.com'}, {'srvr': 'mbox5.example.com'}, {'srvr': 'mbox6.example.com'}])]
            ),
        ]

        for rdata, res in zip(right, results):
            group = _data2rec_group(rschema, rdata, 'prio')
            self.assertEqual(group, res)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class DNSlookupsCase(TestCase):
    '''
    Test the lookup result parsers

    Note that by far and large the parsers actually
    completely ignore the input name or output content
    only nslookup is bad enough to be an exception to that

    a lookup function
        - raises ValueError when an incorrect DNS type is given
        - returns False upon error
        - returns [*record-data] upon succes/no records

    '''
    CMD_RET = {
        'pid': 12345,
        'retcode': 0,
        'stderr': '',
        'stdout': ''
    }

    RESULTS = {
        'A': [
            ['10.1.1.1'],                          # one-match
            ['10.1.1.1', '10.2.2.2', '10.3.3.3'],  # multi-match
        ],
        'AAAA': [
            ['2a00:a00:b01:c02:d03:e04:f05:111'],  # one-match
            ['2a00:a00:b01:c02:d03:e04:f05:111',
             '2a00:a00:b01:c02:d03:e04:f05:222',
             '2a00:a00:b01:c02:d03:e04:f05:333']   # multi-match
        ],
        'CNAME': [
            ['web.example.com.']
        ],
        'MX': [
            ['10 mx1.example.com.'],
            ['10 mx1.example.com.', '20 mx2.example.eu.', '30 mx3.example.nl.']
        ],
        'TXT': [
            ['v=spf1 a include:_spf4.example.com include:mail.example.eu ip4:10.0.0.0/8 ip6:2a00:a00:b01::/48 ~all']
        ]
    }

    def _mock_cmd_ret(self, delta_res):
        '''
        Take CMD_RET and update it w/(a list of ) delta_res
        Mock cmd.run_all w/it
        :param delta_res: list or dict
        :return: patched cmd.run_all
        '''
        if isinstance(delta_res, (list, tuple)):
            test_res = []
            for dres in delta_res:
                tres = self.CMD_RET.copy()
                tres.update(dres)
                test_res.append(tres)

            cmd_mock = MagicMock(
                side_effect=test_res
            )
        else:
            test_res = self.CMD_RET.copy()
            test_res.update(delta_res)
            cmd_mock = MagicMock(
                return_value=test_res
            )

        return patch.dict(salt.utils.dns.__salt__, {'cmd.run_all': cmd_mock}, clear=True)

    def _test_cmd_lookup(self, lookup_cb, wrong_type, wrong, right, empty=None, secure=None):
        '''
        Perform a given battery of tests against a given lookup utilizing cmd.run_all
        :param wrong_type: delta cmd.run_all output for an incorrect DNS type
        :param wrong: delta cmd.run_all output for any error
        :param right: delta cmd.run_all output for outputs in RESULTS
        :param empty: delta cmd.run_all output for anything that won't return matches
        :param secure: delta cmd.run_all output for secured RESULTS
        '''
        # wrong
        for wrong in wrong:
            with self._mock_cmd_ret(wrong):
                self.assertEqual(lookup_cb('mockq', 'A'), False)

        # empty response
        if empty is None:
            empty = {}
        with self._mock_cmd_ret(empty):
            self.assertEqual(lookup_cb('mockq', 'AAAA'), [])

        # wrong types
        with self._mock_cmd_ret(wrong_type):
            self.assertRaises(ValueError, lookup_cb, 'mockq', 'WRONG')

        # Regular outputs
        for rec_t, tests in right.items():
            with self._mock_cmd_ret([dict([('stdout', dres)]) for dres in tests]):
                for test_res in self.RESULTS[rec_t]:
                    if rec_t in ('A', 'AAAA', 'CNAME'):
                        rec = 'mocksrvr.example.com'
                    else:
                        rec = 'example.com'
                    self.assertEqual(
                        lookup_cb(rec, rec_t), test_res,
                        # msg='Error parsing {0} returns'.format(rec_t)
                    )

        if not secure:
            return

        # Regular outputs are insecure outputs (e.g. False)
        for rec_t, tests in right.items():
            with self._mock_cmd_ret([dict([('stdout', dres)]) for dres in tests]):
                for _ in self.RESULTS[rec_t]:
                    self.assertEqual(
                        lookup_cb('mocksrvr.example.com', rec_t, secure=True), False,
                        msg='Insecure {0} returns should not be returned'.format(rec_t)
                    )

        for rec_t, tests in secure.items():
            with self._mock_cmd_ret([dict([('stdout', dres)]) for dres in tests]):
                for test_res in self.RESULTS[rec_t]:
                    self.assertEqual(
                        lookup_cb('mocksrvr.example.com', rec_t, secure=True), test_res,
                        msg='Error parsing DNSSEC\'d {0} returns'.format(rec_t)
                    )

    @skipIf(not salt.utils.dns.HAS_DIG, 'dig is not available')
    def test_dig_options(self):
        cmd = 'dig {0} -v'.format(salt.utils.dns.DIG_OPTIONS)
        cmd = salt.modules.cmdmod.retcode(cmd, python_shell=False, output_loglevel='quiet')
        self.assertEqual(cmd, 0)

    def test_dig(self):
        wrong_type = {'retcode': 0, 'stderr':  ';; Warning, ignoring invalid type ABC'}

        wrongs = [
            {'retcode': 9, 'stderr':  ';; connection timed out; no servers could be reached'},
        ]

        # example returns for dig
        rights = {
            'A': [
                'mocksrvr.example.com.\tA\t10.1.1.1',
                'web.example.com.\t\tA\t10.1.1.1\n'
                'web.example.com.\t\tA\t10.2.2.2\n'
                'web.example.com.\t\tA\t10.3.3.3'

            ],
            'AAAA': [
                'mocksrvr.example.com.\tA\t2a00:a00:b01:c02:d03:e04:f05:111',
                'mocksrvr.example.com.\tCNAME\tweb.example.com.\n'
                'web.example.com.\t\tAAAA\t2a00:a00:b01:c02:d03:e04:f05:111\n'
                'web.example.com.\t\tAAAA\t2a00:a00:b01:c02:d03:e04:f05:222\n'
                'web.example.com.\t\tAAAA\t2a00:a00:b01:c02:d03:e04:f05:333'
            ],
            'CNAME': [
                'mocksrvr.example.com.\tCNAME\tweb.example.com.'
            ],
            'MX': [
                'example.com.\t\tMX\t10 mx1.example.com.',
                'example.com.\t\tMX\t10 mx1.example.com.\nexample.com.\t\tMX\t20 mx2.example.eu.\nexample.com.\t\tMX\t30 mx3.example.nl.'
            ],
            'TXT': [
                'example.com.\tTXT\t"v=spf1 a include:_spf4.example.com include:mail.example.eu ip4:10.0.0.0/8 ip6:2a00:a00:b01::/48 ~all"'
            ]
        }

        secure = {
            'A': [
                'mocksrvr.example.com.\tA\t10.1.1.1\n'
                'mocksrvr.example.com.\tRRSIG\tA 8 3 7200 20170420000000 20170330000000 1629 example.com. Hv4p37EF55LKBxUNYpnhWiEYqfmMct0z0WgDJyG5reqYfl+z4HX/kaoi Wr2iCYuYeB4Le7BgnMSb77UGHPWE7lCQ8z5gkgJ9rCDrooJzSTVdnHfw 1JQ7txRSp8Rj2GLf/L3Ytuo6nNZTV7bWUkfhOs61DAcOPHYZiX8rVhIh UAE=',
                'web.example.com.\t\tA\t10.1.1.1\n'
                'web.example.com.\t\tA\t10.2.2.2\n'
                'web.example.com.\t\tA\t10.3.3.3\n'
                'web.example.com.\tRRSIG\tA 8 3 7200 20170420000000 20170330000000 1629 example.com. Hv4p37EF55LKBxUNYpnhWiEYqfmMct0z0WgDJyG5reqYfl+z4HX/kaoi Wr2iCYuYeB4Le7BgnMSb77UGHPWE7lCQ8z5gkgJ9rCDrooJzSTVdnHfw 1JQ7txRSp8Rj2GLf/L3Ytuo6nNZTV7bWUkfhOs61DAcOPHYZiX8rVhIh UAE='

            ]
        }

        self._test_cmd_lookup(_lookup_dig, wrong=wrongs, right=rights, wrong_type=wrong_type, secure=secure)

    def test_drill(self):
        # all Drill returns look like this
        RES_TMPL = ''';; ->>HEADER<<- opcode: QUERY, rcode: NOERROR, id: 58233
;; flags: qr rd ra ; QUERY: 1, ANSWER: 1, AUTHORITY: 0, ADDITIONAL: 0
;; QUESTION SECTION:
;; mocksrvr.example.com.	IN	A

;; ANSWER SECTION:
{}

;; AUTHORITY SECTION:

;; ADDITIONAL SECTION:

;; Query time: 37 msec
;; SERVER: 10.100.150.129
;; WHEN: Tue Apr  4 19:03:51 2017
;; MSG SIZE  rcvd: 50
'''

        # Not even a different retcode!?
        wrong_type = {'stdout': RES_TMPL.format('mocksrvr.example.com.\t4404\tIN\tA\t10.1.1.1\n')}

        wrongs = [
            {'retcode': 1, 'stderr':  'Error: error sending query: No (valid) nameservers defined in the resolver'}
        ]

        rights = {
            'A': [
                'mocksrvr.example.com.\t4404\tIN\tA\t10.1.1.1\n',
                'web.example.com.\t4404\tIN\tA\t10.1.1.1\n'
                'web.example.com.\t4404\tIN\tA\t10.2.2.2\n'
                'web.example.com.\t4404\tIN\tA\t10.3.3.3',
            ],
            'AAAA': [
                'mocksrvr.example.com.\t4404\tIN\tAAAA\t2a00:a00:b01:c02:d03:e04:f05:111',
                'mocksrvr.example.com.\t4404\tIN\tCNAME\tweb.example.com.\n'
                'web.example.com.\t4404\tIN\tAAAA\t2a00:a00:b01:c02:d03:e04:f05:111\n'
                'web.example.com.\t4404\tIN\tAAAA\t2a00:a00:b01:c02:d03:e04:f05:222\n'
                'web.example.com.\t4404\tIN\tAAAA\t2a00:a00:b01:c02:d03:e04:f05:333'
            ],
            'CNAME': [
                'mocksrvr.example.com.\t4404\tIN\tCNAME\tweb.example.com.'
            ],
            'MX': [
                'example.com.\t4404\tIN\tMX\t10 mx1.example.com.',
                'example.com.\t4404\tIN\tMX\t10 mx1.example.com.\n'
                'example.com.\t4404\tIN\tMX\t20 mx2.example.eu.\n'
                'example.com.\t4404\tIN\tMX\t30 mx3.example.nl.'
            ],
            'TXT': [
                'example.com.\t4404\tIN\tTXT\t"v=spf1 a include:_spf4.example.com include:mail.example.eu ip4:10.0.0.0/8 ip6:2a00:a00:b01::/48 ~all"'
            ]
        }

        secure = {
            'A': [
                'mocksrvr.example.com.\t4404\tIN\tA\t10.1.1.1\n'
                'mocksrvr.example.com.\t4404\tIN\tRRSIG\tA 8 3 7200 20170420000000 20170330000000 1629 example.com. Hv4p37EF55LKBxUNYpnhWiEYqfmMct0z0WgDJyG5reqYfl+z4HX/kaoi Wr2iCYuYeB4Le7BgnMSb77UGHPWE7lCQ8z5gkgJ9rCDrooJzSTVdnHfw 1JQ7txRSp8Rj2GLf/L3Ytuo6nNZTV7bWUkfhOs61DAcOPHYZiX8rVhIh UAE=',
                'web.example.com.\t4404\tIN\tA\t10.1.1.1\n'
                'web.example.com.\t4404\tIN\tA\t10.2.2.2\n'
                'web.example.com.\t4404\tIN\tA\t10.3.3.3\n'
                'web.example.com.\t4404\tIN\tRRSIG\tA 8 3 7200 20170420000000 20170330000000 1629 example.com. Hv4p37EF55LKBxUNYpnhWiEYqfmMct0z0WgDJyG5reqYfl+z4HX/kaoi Wr2iCYuYeB4Le7BgnMSb77UGHPWE7lCQ8z5gkgJ9rCDrooJzSTVdnHfw 1JQ7txRSp8Rj2GLf/L3Ytuo6nNZTV7bWUkfhOs61DAcOPHYZiX8rVhIh UAE='

            ]
        }

        for rec_d in rights, secure:
            for rec_t, tests in rec_d.items():
                for idx, test in enumerate(tests):
                    rec_d[rec_t][idx] = RES_TMPL.format(test)

        self._test_cmd_lookup(_lookup_drill, wrong_type=wrong_type, wrong=wrongs, right=rights, secure=secure)

    def test_gai(self):
        # wrong type
        self.assertRaises(ValueError, _lookup_gai, 'mockq', 'WRONG')

        # wrong
        with patch.object(socket, 'getaddrinfo', MagicMock(side_effect=socket.gaierror)):
            for rec_t in ('A', 'AAAA'):
                self.assertEqual(_lookup_gai('mockq', rec_t), False)

        # example returns from getaddrinfo
        right = {
            'A': [
                [(2, 3, 3, '', ('10.1.1.1', 0))],
                [(2, 3, 3, '', ('10.1.1.1', 0)),
                 (2, 3, 3, '', ('10.2.2.2', 0)),
                 (2, 3, 3, '', ('10.3.3.3', 0))]
            ],
            'AAAA': [
                [(10, 3, 3, '', ('2a00:a00:b01:c02:d03:e04:f05:111', 0, 0, 0))],
                [(10, 3, 3, '', ('2a00:a00:b01:c02:d03:e04:f05:111', 0, 0, 0)),
                 (10, 3, 3, '', ('2a00:a00:b01:c02:d03:e04:f05:222', 0, 0, 0)),
                 (10, 3, 3, '', ('2a00:a00:b01:c02:d03:e04:f05:333', 0, 0, 0))]
            ]
        }

        for rec_t, tests in right.items():
            with patch.object(socket, 'getaddrinfo', MagicMock(side_effect=tests)):
                for test_res in self.RESULTS[rec_t]:
                    self.assertEqual(
                        _lookup_gai('mockq', rec_t), test_res,
                        msg='Error parsing {0} returns'.format(rec_t)
                    )

    def test_host(self):
        wrong_type = {'retcode': 9, 'stderr': 'host: invalid type: WRONG'}

        wrongs = [
            {'retcode': 9, 'stderr': ';; connection timed out; no servers could be reached'}
        ]

        empty = {'stdout': 'www.example.com has no MX record'}

        # example returns for dig
        rights = {
            'A':     [
                'mocksrvr.example.com has address 10.1.1.1',
                'web.example.com  has address 10.1.1.1\n'
                'web.example.com  has address 10.2.2.2\n'
                'web.example.com  has address 10.3.3.3'

            ],
            'AAAA':  [
                'mocksrvr.example.com has IPv6 address 2a00:a00:b01:c02:d03:e04:f05:111',
                'mocksrvr.example.com is an alias for web.example.com.\n'
                'web.example.com has IPv6 address 2a00:a00:b01:c02:d03:e04:f05:111\n'
                'web.example.com has IPv6 address 2a00:a00:b01:c02:d03:e04:f05:222\n'
                'web.example.com has IPv6 address 2a00:a00:b01:c02:d03:e04:f05:333'
            ],
            'CNAME': [
                'mocksrvr.example.com is an alias for web.example.com.'
            ],
            'MX':    [
                'example.com mail is handled by 10 mx1.example.com.',
                'example.com mail is handled by 10 mx1.example.com.\n'
                'example.com mail is handled by 20 mx2.example.eu.\n'
                'example.com mail is handled by 30 mx3.example.nl.'
            ],
            'TXT':   [
                'example.com descriptive text "v=spf1 a include:_spf4.example.com include:mail.example.eu ip4:10.0.0.0/8 ip6:2a00:a00:b01::/48 ~all"'
            ]
        }

        self._test_cmd_lookup(_lookup_host, wrong_type=wrong_type, wrong=wrongs, right=rights, empty=empty)

    def test_dnspython(self):

        pass

    def test_nslookup(self):
        # all nslookup returns look like this
        RES_TMPL = 'Server:\t\t10.11.12.13\nAddress:\t10.11.12.13#53\n\nNon-authoritative answer:\n{}\n\nAuthoritative answers can be found from:'

        wrong_type = {'stdout': 'unknown query type: WRONG' +
                                RES_TMPL.format('Name:\tmocksrvr.example.com\nAddress: 10.1.1.1')}

        wrongs = [
            {'retcode': 1, 'stdout': ';; connection timed out; no servers could be reached'}
        ]

        empty = {'stdout': RES_TMPL.format(
            "*** Can't find www.google.com: No answer\n\nAuthoritative answers can be found from:")}

        rights = {
            'A': [
                'Name:\tmocksrvr.example.com\nAddress: 10.1.1.1',
                'Name:\tmocksrvr.example.com\nAddress: 10.1.1.1\n'
                'Name:\tweb.example.com\nAddress: 10.2.2.2\n'
                'Name:\tweb.example.com\nAddress: 10.3.3.3'
            ],
            'AAAA': [
                'mocksrvr.example.com\thas AAAA address 2a00:a00:b01:c02:d03:e04:f05:111',
                'mocksrvr.example.com\tcanonical name = web.example.com.\n'
                'web.example.com\thas AAAA address 2a00:a00:b01:c02:d03:e04:f05:111\n'
                'web.example.com\thas AAAA address 2a00:a00:b01:c02:d03:e04:f05:222\n'
                'web.example.com\thas AAAA address 2a00:a00:b01:c02:d03:e04:f05:333'
            ],
            'CNAME': [
                'mocksrvr.example.com\tcanonical name = web.example.com.'
            ],
            'MX':    [
                'example.com\tmail exchanger = 10 mx1.example.com.',
                'example.com\tmail exchanger = 10 mx1.example.com.\n'
                'example.com\tmail exchanger = 20 mx2.example.eu.\n'
                'example.com\tmail exchanger = 30 mx3.example.nl.'
            ],
            'TXT':   [
                'example.com\ttext = "v=spf1 a include:_spf4.example.com include:mail.example.eu ip4:10.0.0.0/8 ip6:2a00:a00:b01::/48 ~all"'
            ]
        }

        for rec_t, tests in rights.items():
            for idx, test in enumerate(tests):
                rights[rec_t][idx] = RES_TMPL.format(test)

        self._test_cmd_lookup(_lookup_nslookup, wrong_type=wrong_type, wrong=wrongs, right=rights, empty=empty)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Eric Radman <ericshane@eradman.com>
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    call,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.modules.openbsdpkg as openbsdpkg


@skipIf(NO_MOCK, NO_MOCK_REASON)
class OpenbsdpkgTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.openbsdpkg
    '''
    def setup_loader_modules(self):
        return {openbsdpkg: {}}

    def test_list_pkgs(self):
        '''
        Test for listing installed packages.
        '''
        def _add_data(data, key, value):
            data[key] = value

        pkg_info_out = [
            'png-1.6.23',
            'vim-7.4.1467p1-gtk2',  # vim--gtk2
            'ruby-2.3.1p1'  # ruby%2.3
        ]
        run_stdout_mock = MagicMock(return_value='\n'.join(pkg_info_out))
        patches = {
            'cmd.run_stdout': run_stdout_mock,
            'pkg_resource.add_pkg': _add_data,
            'pkg_resource.sort_pkglist': MagicMock(),
            'pkg_resource.stringify': MagicMock(),
        }
        with patch.dict(openbsdpkg.__salt__, patches):
            pkgs = openbsdpkg.list_pkgs()
            self.assertDictEqual(pkgs, {
                'png': '1.6.23',
                'vim--gtk2': '7.4.1467p1',
                'ruby': '2.3.1p1'})
        run_stdout_mock.assert_called_once_with('pkg_info -q -a',
                                                output_loglevel='trace')

    def test_install_pkgs(self):
        '''
        Test package install behavior for the following conditions:
        - only base package name is given ('png')
        - a flavor is specified ('vim--gtk2')
        - a branch is specified ('ruby%2.3')
        '''
        class ListPackages(object):
            def __init__(self):
                self._iteration = 0

            def __call__(self):
                pkg_lists = [
                     {'vim': '7.4.1467p1-gtk2'},
                     {'png': '1.6.23', 'vim': '7.4.1467p1-gtk2', 'ruby': '2.3.1p1'}
                ]
                pkgs = pkg_lists[self._iteration]
                self._iteration += 1
                return pkgs

        parsed_targets = (
            {'vim--gtk2': None, 'png': None, 'ruby%2.3': None},
            "repository"
        )
        cmd_out = {
            'retcode': 0,
            'stdout': 'quirks-2.241 signed on 2016-07-26T16:56:10Z',
            'stderr': ''
        }
        run_all_mock = MagicMock(return_value=cmd_out)
        patches = {
            'cmd.run_all': run_all_mock,
            'pkg_resource.parse_targets': MagicMock(return_value=parsed_targets),
            'pkg_resource.stringify': MagicMock(),
            'pkg_resource.sort_pkglist': MagicMock(),
        }

        with patch.dict(openbsdpkg.__salt__, patches):
            with patch('salt.modules.openbsdpkg.list_pkgs', ListPackages()):
                added = openbsdpkg.install()
                expected = {
                    'png': {'new': '1.6.23', 'old': ''},
                    'ruby': {'new': '2.3.1p1', 'old': ''}
                }
                self.assertDictEqual(added, expected)
        expected_calls = [
            call('pkg_add -x -I png--%', output_loglevel='trace', python_shell=False),
            call('pkg_add -x -I ruby--%2.3', output_loglevel='trace', python_shell=False),
            call('pkg_add -x -I vim--gtk2%', output_loglevel='trace', python_shell=False),
        ]
        run_all_mock.assert_has_calls(expected_calls, any_order=True)
        self.assertEqual(run_all_mock.call_count, 3)



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.paths import TMP_STATE_TREE


class PillarModuleTest(ModuleCase):
    '''
    Validate the pillar module
    '''
    def test_data(self):
        '''
        pillar.data
        '''
        grains = self.run_function('grains.items')
        pillar = self.run_function('pillar.data')
        self.assertEqual(pillar['os'], grains['os'])
        self.assertEqual(pillar['monty'], 'python')
        if grains['os'] == 'Fedora':
            self.assertEqual(pillar['class'], 'redhat')
        else:
            self.assertEqual(pillar['class'], 'other')

    def test_issue_5449_report_actual_file_roots_in_pillar(self):
        '''
        pillar['master']['file_roots'] is overwritten by the master
        in order to use the fileclient interface to read the pillar
        files. We should restore the actual file_roots when we send
        the pillar back to the minion.
        '''
        self.assertIn(
            TMP_STATE_TREE,
            self.run_function('pillar.data')['master']['file_roots']['base']
        )

    def test_ext_cmd_yaml(self):
        '''
        pillar.data for ext_pillar cmd.yaml
        '''
        self.assertEqual(
                self.run_function('pillar.data')['ext_spam'], 'eggs'
                )

    def test_issue_5951_actual_file_roots_in_opts(self):
        self.assertIn(
            TMP_STATE_TREE,
            self.run_function('pillar.data')['ext_pillar_opts']['file_roots']['base']
        )

    def test_pillar_items(self):
        '''
        Test to ensure we get expected output
        from pillar.items
        '''
        get_items = self.run_function('pillar.items')
        self.assertDictContainsSubset({'monty': 'python'}, get_items)
        self.assertDictContainsSubset(
            {'knights': ['Lancelot', 'Galahad', 'Bedevere', 'Robin']},
            get_items)

    def test_pillar_command_line(self):
        '''
        Test to ensure when using pillar override
        on command line works
        '''
        # test when pillar is overwriting previous pillar
        overwrite = self.run_function('pillar.items', pillar={"monty":
                                                              "overwrite"})
        self.assertDictContainsSubset({'monty': 'overwrite'}, overwrite)

        # test when using additional pillar
        additional = self.run_function('pillar.items', pillar={"new":
                                                              "additional"})

        self.assertDictContainsSubset({'new': 'additional'}, additional)



# -*- coding: utf-8 -*-
'''
    :synopsis: Unit Tests for Windows PKI Module 'module.win_pki'
    :platform: Windows
    :maturity: develop
    .. versionadded:: 2017.7.0
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Libs
import salt.modules.win_pki as win_pki

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON,
)

CERT_PATH = r'C:\certs\testdomain.local.cer'
THUMBPRINT = '9988776655443322111000AAABBBCCCDDDEEEFFF'

CERTS = {
    THUMBPRINT: {
        'dnsnames': ['testdomain.local'],
        'serialnumber': '0123456789AABBCCDD',
        'subject': 'CN=testdomain.local, OU=testou, O=testorg, S=California, C=US',
        'thumbprint': THUMBPRINT,
        'version': 3
    }
}

STORES = {
    'CurrentUser': [
        'AuthRoot',
        'CA',
        'ClientAuthIssuer',
        'Disallowed',
        'MSIEHistoryJournal',
        'My',
        'Root',
        'SmartCardRoot',
        'Trust',
        'TrustedPeople',
        'TrustedPublisher',
        'UserDS'
    ],
    'LocalMachine': [
        'AuthRoot',
        'CA',
        'ClientAuthIssuer',
        'Disallowed',
        'My',
        'Remote Desktop',
        'Root',
        'SmartCardRoot',
        'Trust',
        'TrustedDevices',
        'TrustedPeople',
        'TrustedPublisher',
        'WebHosting'
    ]
}

JSON_CERTS = [{
    'DnsNameList': [{
        'Punycode': 'testdomain.local',
        'Unicode': 'testdomain.local'
    }],
    'SerialNumber': '0123456789AABBCCDD',
    'Subject': 'CN=testdomain.local, OU=testou, O=testorg, S=California, C=US',
    'Thumbprint': '9988776655443322111000AAABBBCCCDDDEEEFFF',
    'Version': 3
}]

JSON_STORES = [{
    'LocationName': 'CurrentUser',
    'StoreNames': STORES['CurrentUser']
}, {
    'LocationName': 'LocalMachine',
    'StoreNames': STORES['LocalMachine']
}]


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinPkiTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.modules.win_pki
    '''
    def setup_loader_modules(self):
        return {win_pki: {}}

    def test_get_stores(self):
        '''
        Test - Get the certificate location contexts and their corresponding stores.
        '''
        with patch.dict(win_pki.__salt__), \
                patch('salt.modules.win_pki._cmd_run',
                      MagicMock(return_value=JSON_STORES)):
            self.assertEqual(win_pki.get_stores(), STORES)

    def test_get_certs(self):
        '''
        Test - Get the available certificates in the given store.
        '''
        with patch.dict(win_pki.__salt__), \
                patch('salt.modules.win_pki._cmd_run',
                      MagicMock(return_value=JSON_CERTS)), \
                patch('salt.modules.win_pki._validate_cert_path',
                      MagicMock(return_value=None)):
            self.assertEqual(win_pki.get_certs(), CERTS)

    def test_get_cert_file(self):
        '''
        Test - Get the details of the certificate file.
        '''
        kwargs = {'name': CERT_PATH}
        with patch.dict(win_pki.__salt__), \
                patch('os.path.isfile',
                     MagicMock(return_value=True)), \
                patch('salt.modules.win_pki._cmd_run',
                      MagicMock(return_value=JSON_CERTS)):
            self.assertEqual(win_pki.get_cert_file(**kwargs), CERTS[THUMBPRINT])

    def test_import_cert(self):
        '''
        Test - Import the certificate file into the given certificate store.
        '''
        kwargs = {'name': CERT_PATH}
        mock_value = MagicMock(return_value=CERT_PATH)
        with patch.dict(win_pki.__salt__, {'cp.cache_file': mock_value}), \
                patch('salt.modules.win_pki._cmd_run',
                      MagicMock(return_value=JSON_CERTS)), \
                patch('salt.modules.win_pki._validate_cert_path',
                      MagicMock(return_value=None)), \
                patch('salt.modules.win_pki.get_cert_file',
                      MagicMock(return_value=CERTS[THUMBPRINT])), \
                patch('salt.modules.win_pki.get_certs',
                      MagicMock(return_value=CERTS)):
            self.assertTrue(win_pki.import_cert(**kwargs))

    def test_export_cert(self):
        '''
        Test - Export the certificate to a file from the given certificate store.
        '''
        kwargs = {'name': CERT_PATH,
                  'thumbprint': THUMBPRINT}
        with patch.dict(win_pki.__salt__), \
                patch('salt.modules.win_pki._cmd_run',
                      MagicMock(return_value='True')), \
                patch('salt.modules.win_pki._validate_cert_format',
                      MagicMock(return_value=None)), \
                patch('salt.modules.win_pki._validate_cert_path',
                      MagicMock(return_value=None)):
            self.assertTrue(win_pki.export_cert(**kwargs))

    def test_test_cert(self):
        '''
        Test - Check the certificate for validity.
        '''
        with patch.dict(win_pki.__salt__), \
                patch('salt.modules.win_pki._cmd_run',
                      MagicMock(return_value='True')), \
                patch('salt.modules.win_pki._validate_cert_path',
                      MagicMock(return_value=None)):
            self.assertTrue(win_pki.test_cert(thumbprint=THUMBPRINT))

    def test_remove_cert(self):
        '''
        Test - Remove the certificate from the given certificate store.
        '''
        with patch.dict(win_pki.__salt__), \
                patch('salt.modules.win_pki._cmd_run',
                      MagicMock(return_value=None)), \
                patch('salt.modules.win_pki._validate_cert_path',
                      MagicMock(return_value=None)), \
                patch('salt.modules.win_pki.get_certs',
                      MagicMock(return_value=CERTS)):
            self.assertTrue(win_pki.remove_cert(thumbprint=THUMBPRINT[::-1]))



# -*- coding: utf-8 -*-
'''
    :codeauthor: Anthony Shaw <anthonyshaw@apache.org>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.netyang as netyang

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch
)

TEST_DATA = {
    'foo': 'bar'
}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class NetyangTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        return {netyang: {}}

    def test_managed(self):
        ret = {'changes': {}, 'comment': 'Loaded.',
               'name': 'test', 'result': False,
               'pchanges': {'compliance_report': {'complies': False}}}
        parse = MagicMock(return_value='abcdef')
        temp_file = MagicMock(return_value='')
        compliance_report = MagicMock(return_value={'complies': False})
        load_config = MagicMock(return_value={'comment': 'Loaded.'})
        file_remove = MagicMock()

        with patch('salt.utils.files.fopen'):
            with patch.dict(netyang.__salt__,
                            {'temp.file': temp_file,
                             'napalm_yang.parse': parse,
                             'napalm_yang.load_config': load_config,
                             'napalm_yang.compliance_report': compliance_report,
                             'file.remove': file_remove}):
                with patch.dict(netyang.__opts__, {'test': False}):
                    self.assertDictEqual(netyang.managed('test', 'test', models=('model1',)), ret)
                    assert parse.called
                    assert temp_file.called
                    assert compliance_report.called
                    assert load_config.called
                    assert file_remove.called

    def test_configured(self):
        ret = {'changes': {}, 'comment': 'Loaded.',
               'name': 'test', 'result': False,
               'pchanges': {}}
        load_config = MagicMock(return_value={'comment': 'Loaded.'})

        with patch('salt.utils.files.fopen'):
            with patch.dict(netyang.__salt__,
                            {'napalm_yang.load_config': load_config}):
                with patch.dict(netyang.__opts__, {'test': False}):
                    self.assertDictEqual(netyang.configured('test', 'test', models=('model1',)), ret)

                    assert load_config.called



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import uuid
import hashlib
import logging
import psutil
import shutil
import signal
import tempfile
import textwrap

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import (
    get_unused_localhost_port,
    skip_if_not_root,
    with_tempfile)
from tests.support.unit import skipIf
import tests.support.paths as paths

# Import salt libs
import salt.utils.files
import salt.utils.path
import salt.utils.platform
import salt.utils.stringutils

log = logging.getLogger(__name__)


class CPModuleTest(ModuleCase):
    '''
    Validate the cp module
    '''
    def run_function(self, *args, **kwargs):
        '''
        Ensure that results are decoded

        TODO: maybe move this behavior to ModuleCase itself?
        '''
        return salt.utils.data.decode(
            super(CPModuleTest, self).run_function(*args, **kwargs)
        )

    @with_tempfile()
    def test_get_file(self, tgt):
        '''
        cp.get_file
        '''
        self.run_function(
                'cp.get_file',
                [
                    'salt://grail/scene33',
                    tgt,
                ])
        with salt.utils.files.fopen(tgt, 'r') as scene:
            data = salt.utils.stringutils.to_unicode(scene.read())
        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
        self.assertNotIn('bacon', data)

    def test_get_file_to_dir(self):
        '''
        cp.get_file
        '''
        tgt = os.path.join(paths.TMP, '')
        self.run_function(
                'cp.get_file',
                [
                    'salt://grail/scene33',
                    tgt,
                ])
        with salt.utils.files.fopen(tgt + 'scene33', 'r') as scene:
            data = salt.utils.stringutils.to_unicode(scene.read())
        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
        self.assertNotIn('bacon', data)

    @with_tempfile()
    def test_get_file_templated_paths(self, tgt):
        '''
        cp.get_file
        '''
        self.run_function(
            'cp.get_file',
            [
                'salt://{{grains.test_grain}}',
                tgt.replace('cheese', '{{grains.test_grain}}')
            ],
            template='jinja'
        )
        with salt.utils.files.fopen(tgt, 'r') as cheese:
            data = salt.utils.stringutils.to_unicode(cheese.read())
        self.assertIn('Gromit', data)
        self.assertNotIn('bacon', data)

    @with_tempfile()
    def test_get_file_gzipped(self, tgt):
        '''
        cp.get_file
        '''
        src = os.path.join(paths.FILES, 'file', 'base', 'file.big')
        with salt.utils.files.fopen(src, 'rb') as fp_:
            hash_str = hashlib.md5(fp_.read()).hexdigest()

        self.run_function(
            'cp.get_file',
            [
                'salt://file.big',
                tgt,
            ],
            gzip=5
        )
        with salt.utils.files.fopen(tgt, 'rb') as scene:
            data = scene.read()
        self.assertEqual(hash_str, hashlib.md5(data).hexdigest())
        data = salt.utils.stringutils.to_unicode(data)
        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
        self.assertNotIn('bacon', data)

    def test_get_file_makedirs(self):
        '''
        cp.get_file
        '''
        tgt = os.path.join(paths.TMP, 'make', 'dirs', 'scene33')
        self.run_function(
            'cp.get_file',
            [
                'salt://grail/scene33',
                tgt,
            ],
            makedirs=True
        )
        self.addCleanup(shutil.rmtree, os.path.join(paths.TMP, 'make'), ignore_errors=True)
        with salt.utils.files.fopen(tgt, 'r') as scene:
            data = salt.utils.stringutils.to_unicode(scene.read())
        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
        self.assertNotIn('bacon', data)

    @with_tempfile()
    def test_get_template(self, tgt):
        '''
        cp.get_template
        '''
        self.run_function(
                'cp.get_template',
                ['salt://grail/scene33', tgt],
                spam='bacon')
        with salt.utils.files.fopen(tgt, 'r') as scene:
            data = salt.utils.stringutils.to_unicode(scene.read())
        self.assertIn('bacon', data)
        self.assertNotIn('spam', data)

    def test_get_dir(self):
        '''
        cp.get_dir
        '''
        tgt = os.path.join(paths.TMP, 'many')
        self.run_function(
                'cp.get_dir',
                [
                    'salt://grail',
                    tgt
                ])
        self.assertIn('grail', os.listdir(tgt))
        self.assertIn('36', os.listdir(os.path.join(tgt, 'grail')))
        self.assertIn('empty', os.listdir(os.path.join(tgt, 'grail')))
        self.assertIn('scene', os.listdir(os.path.join(tgt, 'grail', '36')))

    def test_get_dir_templated_paths(self):
        '''
        cp.get_dir
        '''
        tgt = os.path.join(paths.TMP, 'many')
        self.run_function(
            'cp.get_dir',
            [
                'salt://{{grains.script}}',
                tgt.replace('many', '{{grains.alot}}')
            ]
        )
        self.assertIn('grail', os.listdir(tgt))
        self.assertIn('36', os.listdir(os.path.join(tgt, 'grail')))
        self.assertIn('empty', os.listdir(os.path.join(tgt, 'grail')))
        self.assertIn('scene', os.listdir(os.path.join(tgt, 'grail', '36')))

    # cp.get_url tests

    @with_tempfile()
    def test_get_url(self, tgt):
        '''
        cp.get_url with salt:// source given
        '''
        self.run_function(
            'cp.get_url',
            [
                'salt://grail/scene33',
                tgt,
            ])
        with salt.utils.files.fopen(tgt, 'r') as scene:
            data = salt.utils.stringutils.to_unicode(scene.read())
        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
        self.assertNotIn('bacon', data)

    def test_get_url_makedirs(self):
        '''
        cp.get_url
        '''
        tgt = os.path.join(paths.TMP, 'make', 'dirs', 'scene33')
        self.run_function(
                'cp.get_url',
                [
                    'salt://grail/scene33',
                    tgt,
                ],
                makedirs=True
            )
        self.addCleanup(shutil.rmtree, os.path.join(paths.TMP, 'make'), ignore_errors=True)
        with salt.utils.files.fopen(tgt, 'r') as scene:
            data = salt.utils.stringutils.to_unicode(scene.read())
        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
        self.assertNotIn('bacon', data)

    def test_get_url_dest_empty(self):
        '''
        cp.get_url with salt:// source given and destination omitted.
        '''
        ret = self.run_function(
            'cp.get_url',
            [
                'salt://grail/scene33',
            ])
        with salt.utils.files.fopen(ret, 'r') as scene:
            data = salt.utils.stringutils.to_unicode(scene.read())
        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
        self.assertNotIn('bacon', data)

    def test_get_url_no_dest(self):
        '''
        cp.get_url with salt:// source given and destination set as None
        '''
        tgt = None
        ret = self.run_function(
            'cp.get_url',
            [
                'salt://grail/scene33',
                tgt,
            ])
        self.assertIn('KNIGHT:  They\'re nervous, sire.', ret)

    def test_get_url_nonexistent_source(self):
        '''
        cp.get_url with nonexistent salt:// source given
        '''
        tgt = None
        ret = self.run_function(
            'cp.get_url',
            [
                'salt://grail/nonexistent_scene',
                tgt,
            ])
        self.assertEqual(ret, False)

    def test_get_url_to_dir(self):
        '''
        cp.get_url with salt:// source
        '''
        tgt = os.path.join(paths.TMP, '')
        self.run_function(
                'cp.get_url',
                [
                    'salt://grail/scene33',
                    tgt,
                ])
        with salt.utils.files.fopen(tgt + 'scene33', 'r') as scene:
            data = salt.utils.stringutils.to_unicode(scene.read())
        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
        self.assertNotIn('bacon', data)

    @with_tempfile()
    def test_get_url_https(self, tgt):
        '''
        cp.get_url with https:// source given
        '''
        self.run_function(
            'cp.get_url',
            [
                'https://repo.saltstack.com/index.html',
                tgt,
            ])
        with salt.utils.files.fopen(tgt, 'r') as instructions:
            data = salt.utils.stringutils.to_unicode(instructions.read())
        self.assertIn('Bootstrap', data)
        self.assertIn('Debian', data)
        self.assertIn('Windows', data)
        self.assertNotIn('AYBABTU', data)

    def test_get_url_https_dest_empty(self):
        '''
        cp.get_url with https:// source given and destination omitted.
        '''
        ret = self.run_function(
            'cp.get_url',
            [
                'https://repo.saltstack.com/index.html',
            ])
        with salt.utils.files.fopen(ret, 'r') as instructions:
            data = salt.utils.stringutils.to_unicode(instructions.read())
        self.assertIn('Bootstrap', data)
        self.assertIn('Debian', data)
        self.assertIn('Windows', data)
        self.assertNotIn('AYBABTU', data)

    def test_get_url_https_no_dest(self):
        '''
        cp.get_url with https:// source given and destination set as None
        '''
        tgt = None
        ret = self.run_function(
            'cp.get_url',
            [
                'https://repo.saltstack.com/index.html',
                tgt,
            ])
        self.assertIn('Bootstrap', ret)
        self.assertIn('Debian', ret)
        self.assertIn('Windows', ret)
        self.assertNotIn('AYBABTU', ret)

    def test_get_url_file(self):
        '''
        cp.get_url with file:// source given
        '''
        tgt = ''
        src = os.path.join('file://', paths.FILES, 'file', 'base', 'file.big')
        ret = self.run_function(
            'cp.get_url',
            [
                src,
                tgt,
            ])
        with salt.utils.files.fopen(ret, 'r') as scene:
            data = salt.utils.stringutils.to_unicode(scene.read())
        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
        self.assertNotIn('bacon', data)

    def test_get_url_file_no_dest(self):
        '''
        cp.get_url with file:// source given and destination set as None
        '''
        tgt = None
        src = os.path.join('file://', paths.FILES, 'file', 'base', 'file.big')
        ret = self.run_function(
            'cp.get_url',
            [
                src,
                tgt,
            ])
        self.assertIn('KNIGHT:  They\'re nervous, sire.', ret)
        self.assertNotIn('bacon', ret)

    # cp.get_file_str tests

    def test_get_file_str_salt(self):
        '''
        cp.get_file_str with salt:// source given
        '''
        src = 'salt://grail/scene33'
        ret = self.run_function(
            'cp.get_file_str',
            [
                src,
            ])
        self.assertIn('KNIGHT:  They\'re nervous, sire.', ret)

    def test_get_file_str_nonexistent_source(self):
        '''
        cp.get_file_str with nonexistent salt:// source given
        '''
        src = 'salt://grail/nonexistent_scene'
        ret = self.run_function(
            'cp.get_file_str',
            [
                src,
            ])
        self.assertEqual(ret, False)

    def test_get_file_str_https(self):
        '''
        cp.get_file_str with https:// source given
        '''
        src = 'https://repo.saltstack.com/index.html'
        ret = self.run_function(
            'cp.get_file_str',
            [
                src,
            ])
        self.assertIn('Bootstrap', ret)
        self.assertIn('Debian', ret)
        self.assertIn('Windows', ret)
        self.assertNotIn('AYBABTU', ret)

    def test_get_file_str_local(self):
        '''
        cp.get_file_str with file:// source given
        '''
        src = os.path.join('file://', paths.FILES, 'file', 'base', 'file.big')
        ret = self.run_function(
            'cp.get_file_str',
            [
                src,
            ])
        self.assertIn('KNIGHT:  They\'re nervous, sire.', ret)
        self.assertNotIn('bacon', ret)

    # caching tests

    def test_cache_file(self):
        '''
        cp.cache_file
        '''
        ret = self.run_function(
                'cp.cache_file',
                [
                    'salt://grail/scene33',
                ])
        with salt.utils.files.fopen(ret, 'r') as scene:
            data = salt.utils.stringutils.to_unicode(scene.read())
        self.assertIn('KNIGHT:  They\'re nervous, sire.', data)
        self.assertNotIn('bacon', data)

    def test_cache_files(self):
        '''
        cp.cache_files
        '''
        ret = self.run_function(
                'cp.cache_files',
                [
                    ['salt://grail/scene33', 'salt://grail/36/scene'],
                ])
        for path in ret:
            with salt.utils.files.fopen(path, 'r') as scene:
                data = salt.utils.stringutils.to_unicode(scene.read())
            self.assertIn('ARTHUR:', data)
            self.assertNotIn('bacon', data)

    @with_tempfile()
    def test_cache_master(self, tgt):
        '''
        cp.cache_master
        '''
        ret = self.run_function(
                'cp.cache_master',
                [tgt],
                )
        for path in ret:
            self.assertTrue(os.path.exists(path))

    def test_cache_local_file(self):
        '''
        cp.cache_local_file
        '''
        src = os.path.join(paths.TMP, 'random')
        with salt.utils.files.fopen(src, 'w+') as fn_:
            fn_.write(salt.utils.stringutils.to_str('foo'))
        ret = self.run_function(
                'cp.cache_local_file',
                [src])
        with salt.utils.files.fopen(ret, 'r') as cp_:
            self.assertEqual(
                salt.utils.stringutils.to_unicode(cp_.read()),
                'foo'
            )

    @skipIf(not salt.utils.path.which('nginx'), 'nginx not installed')
    @skip_if_not_root
    def test_cache_remote_file(self):
        '''
        cp.cache_file
        '''
        nginx_port = get_unused_localhost_port()
        url_prefix = 'http://localhost:{0}/'.format(nginx_port)
        temp_dir = tempfile.mkdtemp(dir=paths.TMP)
        self.addCleanup(shutil.rmtree, temp_dir, ignore_errors=True)
        nginx_root_dir = os.path.join(temp_dir, 'root')
        nginx_conf_dir = os.path.join(temp_dir, 'conf')
        nginx_conf = os.path.join(nginx_conf_dir, 'nginx.conf')
        nginx_pidfile = os.path.join(nginx_conf_dir, 'nginx.pid')
        file_contents = 'Hello world!'

        for dirname in (nginx_root_dir, nginx_conf_dir):
            os.mkdir(dirname)

        # Write the temp file
        with salt.utils.files.fopen(os.path.join(nginx_root_dir, 'actual_file'), 'w') as fp_:
            fp_.write(salt.utils.stringutils.to_str(file_contents))

        # Write the nginx config
        with salt.utils.files.fopen(nginx_conf, 'w') as fp_:
            fp_.write(textwrap.dedent(salt.utils.stringutils.to_str(
                '''\
                user root;
                worker_processes 1;
                error_log {nginx_conf_dir}/server_error.log;
                pid {nginx_pidfile};

                events {{
                    worker_connections 1024;
                }}

                http {{
                    include       /etc/nginx/mime.types;
                    default_type  application/octet-stream;

                    access_log {nginx_conf_dir}/access.log;
                    error_log {nginx_conf_dir}/error.log;

                    server {{
                        listen {nginx_port} default_server;
                        server_name cachefile.local;
                        root {nginx_root_dir};

                        location ~ ^/301$ {{
                            return 301 /actual_file;
                        }}

                        location ~ ^/302$ {{
                            return 302 /actual_file;
                        }}
                    }}
                }}'''.format(**locals())
            )))

        self.run_function(
            'cmd.run',
            [['nginx', '-c', nginx_conf]],
            python_shell=False
        )
        with salt.utils.files.fopen(nginx_pidfile) as fp_:
            nginx_pid = int(fp_.read().strip())
            nginx_proc = psutil.Process(pid=nginx_pid)
            self.addCleanup(nginx_proc.send_signal, signal.SIGQUIT)

        for code in ('', '301', '302'):
            url = url_prefix + (code or 'actual_file')
            log.debug('attempting to cache %s', url)
            ret = self.run_function('cp.cache_file', [url])
            self.assertTrue(ret)
            with salt.utils.files.fopen(ret) as fp_:
                cached_contents = salt.utils.stringutils.to_unicode(fp_.read())
                self.assertEqual(cached_contents, file_contents)

    def test_list_states(self):
        '''
        cp.list_states
        '''
        ret = self.run_function(
                'cp.list_states',
                )
        self.assertIn('core', ret)
        self.assertIn('top', ret)

    def test_list_minion(self):
        '''
        cp.list_minion
        '''
        self.run_function(
                'cp.cache_file',
                [
                    'salt://grail/scene33',
                ])
        ret = self.run_function('cp.list_minion')
        found = False
        search = 'grail/scene33'
        if salt.utils.platform.is_windows():
            search = r'grail\scene33'
        for path in ret:
            if search in path:
                found = True
                break
        self.assertTrue(found)

    def test_is_cached(self):
        '''
        cp.is_cached
        '''
        self.run_function(
                'cp.cache_file',
                [
                    'salt://grail/scene33',
                ])
        ret1 = self.run_function(
                'cp.is_cached',
                [
                    'salt://grail/scene33',
                ])
        self.assertTrue(ret1)
        ret2 = self.run_function(
                'cp.is_cached',
                [
                    'salt://fasldkgj/poicxzbn',
                ])
        self.assertFalse(ret2)

    def test_hash_file(self):
        '''
        cp.hash_file
        '''
        sha256_hash = self.run_function(
                'cp.hash_file',
                [
                    'salt://grail/scene33',
                ])
        path = self.run_function(
                'cp.cache_file',
                [
                    'salt://grail/scene33',
                ])
        with salt.utils.files.fopen(path, 'rb') as fn_:
            data = fn_.read()
            self.assertEqual(
                sha256_hash['hsum'], hashlib.sha256(data).hexdigest())

    @with_tempfile()
    def test_get_file_from_env_predefined(self, tgt):
        '''
        cp.get_file
        '''
        tgt = os.path.join(paths.TMP, 'cheese')
        try:
            self.run_function('cp.get_file', ['salt://cheese', tgt])
            with salt.utils.files.fopen(tgt, 'r') as cheese:
                data = salt.utils.stringutils.to_unicode(cheese.read())
            self.assertIn('Gromit', data)
            self.assertNotIn('Comte', data)
        finally:
            os.unlink(tgt)

    @with_tempfile()
    def test_get_file_from_env_in_url(self, tgt):
        tgt = os.path.join(paths.TMP, 'cheese')
        try:
            self.run_function('cp.get_file', ['salt://cheese?saltenv=prod', tgt])
            with salt.utils.files.fopen(tgt, 'r') as cheese:
                data = salt.utils.stringutils.to_unicode(cheese.read())
            self.assertIn('Gromit', data)
            self.assertIn('Comte', data)
        finally:
            os.unlink(tgt)

    def test_push(self):
        log_to_xfer = os.path.join(paths.TMP, uuid.uuid4().hex)
        open(log_to_xfer, 'w').close()  # pylint: disable=resource-leakage
        try:
            self.run_function('cp.push', [log_to_xfer])
            tgt_cache_file = os.path.join(
                    paths.TMP,
                    'master-minion-root',
                    'cache',
                    'minions',
                    'minion',
                    'files',
                    paths.TMP,
                    log_to_xfer)
            self.assertTrue(os.path.isfile(tgt_cache_file), 'File was not cached on the master')
        finally:
            os.unlink(tgt_cache_file)

    def test_envs(self):
        self.assertEqual(sorted(self.run_function('cp.envs')), sorted(['base', 'prod']))



# -*- coding: utf-8 -*-
'''
Tests for the nacl execution module
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import salt.utils.stringutils

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.unit import skipIf

try:
    import libnacl.secret  # pylint: disable=unused-import
    import libnacl.sealed  # pylint: disable=unused-import
    HAS_LIBNACL = True
except ImportError:
    HAS_LIBNACL = False


@skipIf(not HAS_LIBNACL, 'skipping test_nacl, libnacl is unavailable')
class NaclTest(ModuleCase):
    '''
    Test the nacl runner
    '''
    def test_keygen(self):
        '''
        Test keygen
        '''
        # Store the data
        ret = self.run_function(
            'nacl.keygen',
        )
        self.assertIn('pk', ret)
        self.assertIn('sk', ret)

    def test_enc_dec(self):
        '''
        Generate keys, encrypt, then decrypt.
        '''
        # Store the data
        ret = self.run_function(
            'nacl.keygen',
        )
        self.assertIn('pk', ret)
        self.assertIn('sk', ret)
        pk = ret['pk']
        sk = ret['sk']

        unencrypted_data = salt.utils.stringutils.to_str('hello')

        # Encrypt with pk
        ret = self.run_function(
            'nacl.enc',
            data=unencrypted_data,
            pk=pk,
        )
        encrypted_data = ret

        # Decrypt with sk
        ret = self.run_function(
            'nacl.dec',
            data=encrypted_data,
            sk=sk,
        )
        self.assertEqual(unencrypted_data, ret)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.integration.modules.event
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import python libs
from __future__ import absolute_import
import time
import threading

# Import Salt Testing libs
from tests.support.case import ModuleCase

# Import salt libs
import salt.utils.event as event

# Import 3rd-party libs
from salt.ext.six.moves.queue import Queue, Empty  # pylint: disable=import-error,no-name-in-module


class EventModuleTest(ModuleCase):
    def __test_event_fire_master(self):
        events = Queue()

        def get_event(events):
            me = event.MasterEvent(self.master_opts['sock_dir'], listen=True)
            events.put_nowait(
                me.get_event(wait=10, tag='salttest', full=False)
            )

        threading.Thread(target=get_event, args=(events,)).start()
        time.sleep(1)   # Allow multiprocessing.Process to start

        ret = self.run_function(
            'event.fire_master',
            ['event.fire_master: just test it!!!!', 'salttest']
        )
        self.assertTrue(ret)

        eventfired = events.get(block=True, timeout=10)
        self.assertIsNotNone(eventfired)
        self.assertIn(
            'event.fire_master: just test it!!!!', eventfired['data']
        )

        ret = self.run_function(
            'event.fire_master',
            ['event.fire_master: just test it!!!!', 'salttest-miss']
        )
        self.assertTrue(ret)

        with self.assertRaises(Empty):
            eventfired = events.get(block=True, timeout=10)

    def __test_event_fire(self):
        events = Queue()

        def get_event(events):
            me = event.MinionEvent(self.minion_opts, listen=True)
            events.put_nowait(
                me.get_event(wait=10, tag='salttest', full=False)
            )

        threading.Thread(target=get_event, args=(events,)).start()
        time.sleep(1)   # Allow multiprocessing.Process to start

        ret = self.run_function(
            'event.fire', ['event.fire: just test it!!!!', 'salttest']
        )
        self.assertTrue(ret)

        eventfired = events.get(block=True, timeout=10)
        self.assertIsNotNone(eventfired)
        self.assertIn('event.fire: just test it!!!!', eventfired)

        ret = self.run_function(
            'event.fire', ['event.fire: just test it!!!!', 'salttest-miss']
        )
        self.assertTrue(ret)

        with self.assertRaises(Empty):
            eventfired = events.get(block=True, timeout=10)

    def __test_event_fire_ipc_mode_tcp(self):
        events = Queue()

        def get_event(events):
            me = event.MinionEvent(self.sub_minion_opts, listen=True)
            events.put_nowait(
                me.get_event(wait=10, tag='salttest', full=False)
            )

        threading.Thread(target=get_event, args=(events,)).start()
        time.sleep(1)   # Allow multiprocessing.Process to start

        ret = self.run_function(
            'event.fire', ['event.fire: just test it!!!!', 'salttest'],
            minion_tgt='sub_minion'
        )
        self.assertTrue(ret)

        eventfired = events.get(block=True, timeout=10)
        self.assertIsNotNone(eventfired)
        self.assertIn('event.fire: just test it!!!!', eventfired)

        ret = self.run_function(
            'event.fire', ['event.fire: just test it!!!!', 'salttest-miss'],
            minion_tgt='sub_minion'
        )
        self.assertTrue(ret)

        with self.assertRaises(Empty):
            eventfired = events.get(block=True, timeout=10)



# -*- coding: utf-8 -*-
'''
    Unit tests for salt.utils.yamlloader.SaltYamlSafeLoader
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import collections
import textwrap

# Import Salt Libs
from yaml.constructor import ConstructorError
from salt.utils.yamlloader import SaltYamlSafeLoader
import salt.utils.files
from salt.ext import six

# Import Salt Testing Libs
from tests.support.unit import TestCase, skipIf
from tests.support.mock import patch, NO_MOCK, NO_MOCK_REASON, mock_open

# Import 3rd-party libs
from salt.ext import six


@skipIf(NO_MOCK, NO_MOCK_REASON)
class YamlLoaderTestCase(TestCase):
    '''
    TestCase for salt.utils.yamlloader module
    '''

    @staticmethod
    def render_yaml(data):
        '''
        Takes a YAML string, puts it into a mock file, passes that to the YAML
        SaltYamlSafeLoader and then returns the rendered/parsed YAML data
        '''
        if six.PY2:
            # On Python 2, data read from a filehandle will not already be
            # unicode, so we need to encode it first to properly simulate
            # reading from a file. This is because unicode_literals is imported
            # and all of the data to be used in mock_open will be a unicode
            # type. Encoding will make it a str.
            data = salt.utils.data.encode(data)
        with patch('salt.utils.files.fopen', mock_open(read_data=data)) as mocked_file:
            with salt.utils.files.fopen(mocked_file) as mocked_stream:
                return SaltYamlSafeLoader(mocked_stream).get_data()

    @staticmethod
    def raise_error(value):
        raise TypeError('{0!r} is not a unicode string'.format(value))  # pylint: disable=repr-flag-used-in-string

    def assert_unicode(self, value):
        '''
        Make sure the entire data structure is unicode
        '''
        if six.PY3:
            return
        if isinstance(value, six.string_types):
            if not isinstance(value, six.text_type):
                self.raise_error(value)
        elif isinstance(value, collections.Mapping):
            for k, v in six.iteritems(value):
                self.assert_unicode(k)
                self.assert_unicode(v)
        elif isinstance(value, collections.Iterable):
            for item in value:
                self.assert_unicode(item)

    def assert_matches(self, ret, expected):
        self.assertEqual(ret, expected)
        self.assert_unicode(ret)

    def test_yaml_basics(self):
        '''
        Test parsing an ordinary path
        '''
        self.assert_matches(
            self.render_yaml(textwrap.dedent('''\
                p1:
                  - alpha
                  - beta''')),
            {'p1': ['alpha', 'beta']}
        )

    def test_yaml_merge(self):
        '''
        Test YAML anchors
        '''
        # Simple merge test
        self.assert_matches(
            self.render_yaml(textwrap.dedent('''\
                p1: &p1
                  v1: alpha
                p2:
                  <<: *p1
                  v2: beta''')),
            {'p1': {'v1': 'alpha'}, 'p2': {'v1': 'alpha', 'v2': 'beta'}}
        )

        # Test that keys/nodes are overwritten
        self.assert_matches(
            self.render_yaml(textwrap.dedent('''\
                p1: &p1
                  v1: alpha
                p2:
                  <<: *p1
                  v1: new_alpha''')),
            {'p1': {'v1': 'alpha'}, 'p2': {'v1': 'new_alpha'}}
        )

        # Test merging of lists
        self.assert_matches(
            self.render_yaml(textwrap.dedent('''\
                p1: &p1
                  v1: &v1
                    - t1
                    - t2
                p2:
                  v2: *v1''')),
            {"p2": {"v2": ["t1", "t2"]}, "p1": {"v1": ["t1", "t2"]}}
        )

    def test_yaml_duplicates(self):
        '''
        Test that duplicates still throw an error
        '''
        with self.assertRaises(ConstructorError):
            self.render_yaml(textwrap.dedent('''\
                p1: alpha
                p1: beta'''))

        with self.assertRaises(ConstructorError):
            self.render_yaml(textwrap.dedent('''\
                p1: &p1
                  v1: alpha
                p2:
                  <<: *p1
                  v2: beta
                  v2: betabeta'''))

    def test_yaml_with_unicode_literals(self):
        '''
        Test proper loading of unicode literals
        '''
        self.assert_matches(
            self.render_yaml(textwrap.dedent('''\
                foo:
                  a: Д
                  b: {'a': u'\\u0414'}''')),
            {'foo': {'a': u'\u0414', 'b': {'a': u'\u0414'}}}
        )

    def test_yaml_with_colon_in_inline_dict(self):
        '''
        Test proper loading of unicode literal strings in inline dicts
        '''
        self.assert_matches(
            self.render_yaml(textwrap.dedent('''\
                foo:
                  b: {u'c': u'https://foo.com'}''')),
            {'foo': {'b': {'c': 'https://foo.com'}}}
        )

    def test_yaml_with_plain_scalars(self):
        '''
        Test that plain (i.e. unqoted) string and non-string scalars are
        properly handled
        '''
        self.assert_matches(
            self.render_yaml(textwrap.dedent('''\
                foo:
                  b: {foo: bar, one: 1, list: [1, two, 3]}''')),
            {'foo': {'b': {'foo': 'bar', 'one': 1, 'list': [1, 'two', 3]}}}
        )



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)

import tests.support.napalm as napalm_test_support
import salt.modules.napalm_acl as napalm_acl  # NOQA


def mock_capirca_term_config(platform, filter_name, term_name, *args, **kwargs):
    assert platform == 'cisco'
    assert filter_name == 'test_filter'
    assert term_name == 'test_term'
    return 'test_config'


def mock_capirca_filter_config(platform, filter_name, *args, **kwargs):
    assert platform == 'cisco'
    assert filter_name == 'test_filter'
    return 'test_config'


def mock_capirca_policy_config(platform, *args, **kwargs):
    assert platform == 'cisco'
    return 'test_config'


def mock_net_load_config(text, *args, **kwargs):
    assert text == 'test_config'
    return napalm_test_support.TEST_TERM_CONFIG


def mock_capirca_get_filter_pillar(filter_, *args, **kwargs):
    assert filter_ == 'test_filter'
    return {'test': 'value'}


def mock_capirca_get_term_pillar(filter_, term, *args, **kwargs):
    assert filter_ == 'test_filter'
    assert term == 'test_term'
    return {'test': 'value'}


@skipIf(NO_MOCK, NO_MOCK_REASON)
class NapalmAclModuleTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        module_globals = {
            '__salt__': {
                'config.option': MagicMock(return_value={
                    'test': {
                        'driver': 'test',
                        'key': '2orgk34kgk34g'
                    }
                }),
                'file.file_exists': napalm_test_support.true,
                'file.join': napalm_test_support.join,
                'file.get_managed': napalm_test_support.get_managed_file,
                'random.hash': napalm_test_support.random_hash,
                'capirca.get_term_config': mock_capirca_term_config,
                'capirca.get_policy_config': mock_capirca_policy_config,
                'capirca.get_filter_config': mock_capirca_filter_config,
                'capirca.get_filter_pillar': mock_capirca_get_filter_pillar,
                'capirca.get_term_pillar': mock_capirca_get_term_pillar,
                'net.load_config': mock_net_load_config
            },
            '__grains__': {
                'os': 'ios',
                'vendor': 'cisco',
                'model': '3750X'
            }
        }

        return {napalm_acl: module_globals}

    def test_load_term_config(self):
        ret = napalm_acl.load_term_config('test_filter', 'test_term')
        assert ret['already_configured'] is False

    def test_load_filter_config(self):
        ret = napalm_acl.load_filter_config('test_filter', 'test_term')
        assert ret['already_configured'] is False

    def test_load_policy_config(self):
        ret = napalm_acl.load_policy_config('test_filter', 'test_term')
        assert ret['already_configured'] is False

    def test_get_filter_pillar(self):
        ret = napalm_acl.get_filter_pillar('test_filter')
        assert ret['test'] == 'value'

    def test_get_term_pillar(self):
        ret = napalm_acl.get_term_pillar('test_filter', 'test_term')
        assert ret['test'] == 'value'



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Libs
import salt.states.win_wusa as wusa
from salt.exceptions import SaltInvocationError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase
from tests.support.mock import MagicMock, patch


class WinWusaTestCase(TestCase, LoaderModuleMockMixin):
    '''
    test the function in the win_wusa state module
    '''
    kb = 'KB123456'

    def setup_loader_modules(self):
        return {wusa: {'__opts__': {'test': False},
                       '__env__': 'base'}}

    def test_installed_no_source(self):
        '''
        test wusa.installed without passing source
        '''
        with self.assertRaises(SaltInvocationError) as excinfo:
            wusa.installed(name='KB123456', source=None)

        self.assertEqual(excinfo.exception.strerror,
                         'Must specify a "source" file to install')

    def test_installed_existing(self):
        '''
        test wusa.installed when the kb is already installed
        '''
        mock_installed = MagicMock(return_value=True)
        with patch.dict(wusa.__salt__, {'wusa.is_installed': mock_installed}):
            returned = wusa.installed(name=self.kb,
                                      source='salt://{0}.msu'.format(self.kb))
            expected = {'changes': {},
                        'comment': '{0} already installed'.format(self.kb),
                        'name': self.kb,
                        'result': True}
            self.assertDictEqual(expected, returned)

    def test_installed_test_true(self):
        '''
        test wusa.installed with test=True
        '''
        mock_installed = MagicMock(return_value=False)
        with patch.dict(wusa.__salt__, {'wusa.is_installed': mock_installed}), \
                patch.dict(wusa.__opts__, {'test': True}):
            returned = wusa.installed(name=self.kb,
                                      source='salt://{0}.msu'.format(self.kb))
            expected = {'changes': {},
                        'comment': '{0} would be installed'.format(self.kb),
                        'name': self.kb,
                        'result': None}
            self.assertDictEqual(expected, returned)

    def test_installed_cache_fail(self):
        '''
        test wusa.install when it fails to cache the file
        '''
        mock_installed = MagicMock(return_value=False)
        mock_cache = MagicMock(return_value='')
        with patch.dict(wusa.__salt__, {'wusa.is_installed': mock_installed,
                                        'cp.cache_file': mock_cache}):
            returned = wusa.installed(name=self.kb,
                                      source='salt://{0}.msu'.format(self.kb))
            expected = {'changes': {},
                        'comment': 'Unable to cache salt://{0}.msu from '
                                   'saltenv "base"'.format(self.kb),
                        'name': self.kb,
                        'result': False}
            self.assertDictEqual(expected, returned)

    def test_installed(self):
        '''
        test wusa.installed assuming success
        '''
        mock_installed = MagicMock(side_effect=[False, True])
        mock_cache = MagicMock(return_value='C:\\{0}.msu'.format(self.kb))
        with patch.dict(wusa.__salt__, {'wusa.is_installed': mock_installed,
                                        'cp.cache_file': mock_cache,
                                        'wusa.install': MagicMock()}):
            returned = wusa.installed(name=self.kb,
                                      source='salt://{0}.msu'.format(self.kb))
            expected = {'changes': {'new': True, 'old': False},
                        'comment': '{0} was installed'.format(self.kb),
                        'name': self.kb,
                        'result': True}
            self.assertDictEqual(expected, returned)

    def test_installed_failed(self):
        '''
        test wusa.installed with a failure
        '''
        mock_installed = MagicMock(side_effect=[False, False])
        mock_cache = MagicMock(return_value='C:\\{0}.msu'.format(self.kb))
        with patch.dict(wusa.__salt__, {'wusa.is_installed': mock_installed,
                                        'cp.cache_file': mock_cache,
                                        'wusa.install': MagicMock()}):
            returned = wusa.installed(name=self.kb,
                                      source='salt://{0}.msu'.format(self.kb))
            expected = {'changes': {},
                        'comment': '{0} failed to install'.format(self.kb),
                        'name': self.kb,
                        'result': False}
            self.assertDictEqual(expected, returned)

    def test_uninstalled_non_existing(self):
        '''
        test wusa.uninstalled when the kb is not installed
        '''
        mock_installed = MagicMock(return_value=False)
        with patch.dict(wusa.__salt__, {'wusa.is_installed': mock_installed}):
            returned = wusa.uninstalled(name=self.kb)
            expected = {'changes': {},
                        'comment': '{0} already uninstalled'.format(self.kb),
                        'name': self.kb,
                        'result': True}
            self.assertDictEqual(expected, returned)

    def test_uninstalled_test_true(self):
        '''
        test wusa.uninstalled with test=True
        '''
        mock_installed = MagicMock(return_value=True)
        with patch.dict(wusa.__salt__, {'wusa.is_installed': mock_installed}), \
                patch.dict(wusa.__opts__, {'test': True}):
            returned = wusa.uninstalled(name=self.kb)
            expected = {'changes': {},
                        'comment': '{0} would be uninstalled'.format(self.kb),
                        'name': self.kb,
                        'result': None}
            self.assertDictEqual(expected, returned)

    def test_uninstalled(self):
        '''
        test wusa.uninstalled assuming success
        '''
        mock_installed = MagicMock(side_effect=[True, False])
        with patch.dict(wusa.__salt__, {'wusa.is_installed': mock_installed,
                                        'wusa.uninstall': MagicMock()}):
            returned = wusa.uninstalled(name=self.kb)
            expected = {'changes': {'new': False, 'old': True},
                        'comment': '{0} was uninstalled'.format(self.kb),
                        'name': self.kb,
                        'result': True}
            self.assertDictEqual(expected, returned)

    def test_uninstalled_failed(self):
        '''
        test wusa.uninstalled with a failure
        '''
        mock_installed = MagicMock(side_effect=[True, True])
        with patch.dict(wusa.__salt__, {'wusa.is_installed': mock_installed,
                                        'wusa.uninstall': MagicMock()}):
            returned = wusa.uninstalled(name=self.kb)
            expected = {'changes': {},
                        'comment': '{0} failed to uninstall'.format(self.kb),
                        'name': self.kb,
                        'result': False}
            self.assertDictEqual(expected, returned)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import shutil
import tempfile

# Import Salt Testing libs
import tests.integration as integration
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)
from tests.support.mixins import AdaptedConfigurationTestCaseMixin
from tests.support.paths import BASE_FILES

# Import Salt libs
import salt.exceptions
import salt.state
from salt.utils.odict import OrderedDict
from salt.utils.decorators import state as statedecorators

try:
    import pytest
except ImportError as err:
    pytest = None


@skipIf(NO_MOCK, NO_MOCK_REASON)
class StateCompilerTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
    '''
    TestCase for the state compiler.
    '''

    def test_format_log_non_ascii_character(self):
        '''
        Tests running a non-ascii character through the state.format_log
        function. See Issue #33605.
        '''
        # There is no return to test against as the format_log
        # function doesn't return anything. However, we do want
        # to make sure that the function doesn't stacktrace when
        # called.
        ret = {'changes': {'Français': {'old': 'something old',
                                         'new': 'something new'}},
               'result': True}
        salt.state.format_log(ret)

    def test_render_error_on_invalid_requisite(self):
        '''
        Test that the state compiler correctly deliver a rendering
        exception when a requisite cannot be resolved
        '''
        with patch('salt.state.State._gather_pillar') as state_patch:
            high_data = {
                'git': OrderedDict([
                    ('pkg', [
                        OrderedDict([
                            ('require', [
                                OrderedDict([
                                    ('file', OrderedDict(
                                        [('test1', 'test')]))])])]),
                        'installed', {'order': 10000}]), ('__sls__', 'issue_35226'), ('__env__', 'base')])}
            minion_opts = self.get_temp_config('minion')
            minion_opts['pillar'] = {'git': OrderedDict([('test1', 'test')])}
            state_obj = salt.state.State(minion_opts)
            with self.assertRaises(salt.exceptions.SaltRenderError):
                state_obj.call_high(high_data)


class HighStateTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
    def setUp(self):
        root_dir = tempfile.mkdtemp(dir=integration.TMP)
        self.state_tree_dir = os.path.join(root_dir, 'state_tree')
        cache_dir = os.path.join(root_dir, 'cachedir')
        for dpath in (root_dir, self.state_tree_dir, cache_dir):
            if not os.path.isdir(dpath):
                os.makedirs(dpath)

        overrides = {}
        overrides['root_dir'] = root_dir
        overrides['state_events'] = False
        overrides['id'] = 'match'
        overrides['file_client'] = 'local'
        overrides['file_roots'] = dict(base=[self.state_tree_dir])
        overrides['cachedir'] = cache_dir
        overrides['test'] = False
        self.config = self.get_temp_config('minion', **overrides)
        self.addCleanup(delattr, self, 'config')
        self.highstate = salt.state.HighState(self.config)
        self.addCleanup(delattr, self, 'highstate')
        self.highstate.push_active()

    def tearDown(self):
        self.highstate.pop_active()

    def test_top_matches_with_list(self):
        top = {'env': {'match': ['state1', 'state2'], 'nomatch': ['state3']}}
        matches = self.highstate.top_matches(top)
        self.assertEqual(matches, {'env': ['state1', 'state2']})

    def test_top_matches_with_string(self):
        top = {'env': {'match': 'state1', 'nomatch': 'state2'}}
        matches = self.highstate.top_matches(top)
        self.assertEqual(matches, {'env': ['state1']})

    def test_matches_whitelist(self):
        matches = {'env': ['state1', 'state2', 'state3']}
        matches = self.highstate.matches_whitelist(matches, ['state2'])
        self.assertEqual(matches, {'env': ['state2']})

    def test_matches_whitelist_with_string(self):
        matches = {'env': ['state1', 'state2', 'state3']}
        matches = self.highstate.matches_whitelist(matches,
                                                   'state2,state3')
        self.assertEqual(matches, {'env': ['state2', 'state3']})

    def test_show_state_usage(self):
        # monkey patch sub methods
        self.highstate.avail = {
            'base': ['state.a', 'state.b', 'state.c']
        }

        def verify_tops(*args, **kwargs):
            return []

        def get_top(*args, **kwargs):
            return None

        def top_matches(*args, **kwargs):
            return {'base': ['state.a', 'state.b']}

        self.highstate.verify_tops = verify_tops
        self.highstate.get_top = get_top
        self.highstate.top_matches = top_matches

        # get compile_state_usage() result
        state_usage_dict = self.highstate.compile_state_usage()

        self.assertEqual(state_usage_dict['base']['count_unused'], 1)
        self.assertEqual(state_usage_dict['base']['count_used'], 2)
        self.assertEqual(state_usage_dict['base']['count_all'], 3)
        self.assertEqual(state_usage_dict['base']['used'], ['state.a', 'state.b'])
        self.assertEqual(state_usage_dict['base']['unused'], ['state.c'])

    def test_find_sls_ids_with_exclude(self):
        '''
        See https://github.com/saltstack/salt/issues/47182
        '''
        sls_dir = 'issue-47182'
        shutil.copytree(
            os.path.join(BASE_FILES, sls_dir),
            os.path.join(self.state_tree_dir, sls_dir)
        )
        shutil.move(
            os.path.join(self.state_tree_dir, sls_dir, 'top.sls'),
            self.state_tree_dir
        )
        # Manually compile the high data. We don't have to worry about all of
        # the normal error checking we do here since we know that all the SLS
        # files exist and there is no whitelist/blacklist being used.
        top = self.highstate.get_top()  # pylint: disable=assignment-from-none
        matches = self.highstate.top_matches(top)
        high, _ = self.highstate.render_highstate(matches)
        ret = salt.state.find_sls_ids('issue-47182.stateA.newer', high)
        self.assertEqual(ret, [('somestuff', 'cmd')])


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(pytest is None, 'PyTest is missing')
class StateReturnsTestCase(TestCase):
    '''
    TestCase for code handling state returns.
    '''

    def test_state_output_check_changes_is_dict(self):
        '''
        Test that changes key contains a dictionary.
        :return:
        '''
        data = {'changes': []}
        out = statedecorators.OutputUnifier('content_check')(lambda: data)()
        assert "'Changes' should be a dictionary" in out['comment']
        assert not out['result']

    def test_state_output_check_return_is_dict(self):
        '''
        Test for the entire return is a dictionary
        :return:
        '''
        data = ['whatever']
        out = statedecorators.OutputUnifier('content_check')(lambda: data)()
        assert 'Malformed state return. Data must be a dictionary type' in out['comment']
        assert not out['result']

    def test_state_output_check_return_has_nrc(self):
        '''
        Test for name/result/comment keys are inside the return.
        :return:
        '''
        data = {'arbitrary': 'data', 'changes': {}}
        out = statedecorators.OutputUnifier('content_check')(lambda: data)()
        assert ' The following keys were not present in the state return: name, result, comment' in out['comment']
        assert not out['result']

    def test_state_output_unifier_comment_is_not_list(self):
        '''
        Test for output is unified so the comment is converted to a multi-line string
        :return:
        '''
        data = {'comment': ['data', 'in', 'the', 'list'], 'changes': {}, 'name': None, 'result': 'fantastic!'}
        expected = {'comment': 'data\nin\nthe\nlist', 'changes': {}, 'name': None, 'result': True}
        assert statedecorators.OutputUnifier('unify')(lambda: data)() == expected

        data = {'comment': ['data', 'in', 'the', 'list'], 'changes': {}, 'name': None, 'result': None}
        expected = 'data\nin\nthe\nlist'
        assert statedecorators.OutputUnifier('unify')(lambda: data)()['comment'] == expected

    def test_state_output_unifier_result_converted_to_true(self):
        '''
        Test for output is unified so the result is converted to True
        :return:
        '''
        data = {'comment': ['data', 'in', 'the', 'list'], 'changes': {}, 'name': None, 'result': 'Fantastic'}
        assert statedecorators.OutputUnifier('unify')(lambda: data)()['result'] is True

    def test_state_output_unifier_result_converted_to_false(self):
        '''
        Test for output is unified so the result is converted to False
        :return:
        '''
        data = {'comment': ['data', 'in', 'the', 'list'], 'changes': {}, 'name': None, 'result': ''}
        assert statedecorators.OutputUnifier('unify')(lambda: data)()['result'] is False


class StateFormatSlotsTestCase(TestCase, AdaptedConfigurationTestCaseMixin):
    '''
    TestCase for code handling slots
    '''
    def setUp(self):
        with patch('salt.state.State._gather_pillar'):
            minion_opts = self.get_temp_config('minion')
            self.state_obj = salt.state.State(minion_opts)

    def test_format_slots_no_slots(self):
        '''
        Test the format slots keeps data without slots untouched.
        '''
        cdata = {
                'args': [
                    'arg',
                ],
                'kwargs': {
                    'key': 'val',
                }
        }
        self.state_obj.format_slots(cdata)
        self.assertEqual(cdata, {'args': ['arg'], 'kwargs': {'key': 'val'}})

    def test_format_slots_arg(self):
        '''
        Test the format slots is calling a slot specified in args with corresponding arguments.
        '''
        cdata = {
                'args': [
                    '__slot__:salt:mod.fun(fun_arg, fun_key=fun_val)',
                ],
                'kwargs': {
                    'key': 'val',
                }
        }
        mock = MagicMock(return_value='fun_return')
        with patch.dict(self.state_obj.functions, {'mod.fun': mock}):
            self.state_obj.format_slots(cdata)
        mock.assert_called_once_with('fun_arg', fun_key='fun_val')
        self.assertEqual(cdata, {'args': ['fun_return'], 'kwargs': {'key': 'val'}})

    def test_format_slots_kwarg(self):
        '''
        Test the format slots is calling a slot specified in kwargs with corresponding arguments.
        '''
        cdata = {
            'args': [
                'arg',
            ],
            'kwargs': {
                'key': '__slot__:salt:mod.fun(fun_arg, fun_key=fun_val)',
            }
        }
        mock = MagicMock(return_value='fun_return')
        with patch.dict(self.state_obj.functions, {'mod.fun': mock}):
            self.state_obj.format_slots(cdata)
        mock.assert_called_once_with('fun_arg', fun_key='fun_val')
        self.assertEqual(cdata, {'args': ['arg'], 'kwargs': {'key': 'fun_return'}})

    def test_format_slots_multi(self):
        '''
        Test the format slots is calling all slots with corresponding arguments when multiple slots
        specified.
        '''
        cdata = {
            'args': [
                '__slot__:salt:test_mod.fun_a(a_arg, a_key=a_kwarg)',
                '__slot__:salt:test_mod.fun_b(b_arg, b_key=b_kwarg)',
            ],
            'kwargs': {
                'kw_key_1': '__slot__:salt:test_mod.fun_c(c_arg, c_key=c_kwarg)',
                'kw_key_2': '__slot__:salt:test_mod.fun_d(d_arg, d_key=d_kwarg)',
            }
        }
        mock_a = MagicMock(return_value='fun_a_return')
        mock_b = MagicMock(return_value='fun_b_return')
        mock_c = MagicMock(return_value='fun_c_return')
        mock_d = MagicMock(return_value='fun_d_return')
        with patch.dict(self.state_obj.functions, {'test_mod.fun_a': mock_a,
                                                   'test_mod.fun_b': mock_b,
                                                   'test_mod.fun_c': mock_c,
                                                   'test_mod.fun_d': mock_d}):
            self.state_obj.format_slots(cdata)
        mock_a.assert_called_once_with('a_arg', a_key='a_kwarg')
        mock_b.assert_called_once_with('b_arg', b_key='b_kwarg')
        mock_c.assert_called_once_with('c_arg', c_key='c_kwarg')
        mock_d.assert_called_once_with('d_arg', d_key='d_kwarg')
        self.assertEqual(cdata, {'args': ['fun_a_return',
                                          'fun_b_return'],
                                 'kwargs': {'kw_key_1': 'fun_c_return',
                                            'kw_key_2': 'fun_d_return'}})

    def test_format_slots_malformed(self):
        '''
        Test the format slots keeps malformed slots untouched.
        '''
        sls_data = {
            'args': [
                '__slot__:NOT_SUPPORTED:not.called()',
                '__slot__:salt:not.called(',
                '__slot__:salt:',
                '__slot__:salt',
                '__slot__:',
                '__slot__',
            ],
            'kwargs': {
                'key3': '__slot__:NOT_SUPPORTED:not.called()',
                'key4': '__slot__:salt:not.called(',
                'key5': '__slot__:salt:',
                'key6': '__slot__:salt',
                'key7': '__slot__:',
                'key8': '__slot__',
            }
        }
        cdata = sls_data.copy()
        mock = MagicMock(return_value='return')
        with patch.dict(self.state_obj.functions, {'not.called': mock}):
            self.state_obj.format_slots(cdata)
        mock.assert_not_called()
        self.assertEqual(cdata, sls_data)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Ted Strzalkowski (tedski@gmail.com)


    tests.unit.modules.mdadm_test
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, MagicMock, patch

# Import salt libs
import salt.modules.mdadm as mdadm


@skipIf(NO_MOCK, NO_MOCK_REASON)
class MdadmTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {mdadm: {}}

    def test_create(self):
        mock = MagicMock(return_value='salt')
        with patch.dict(mdadm.__salt__, {'cmd.run': mock}), \
                patch('salt.utils.path.which', lambda exe: exe):
            ret = mdadm.create(
                    '/dev/md0', 5,
                    devices=['/dev/sdb1', '/dev/sdc1', '/dev/sdd1'],
                    test_mode=False,
                    force=True,
                    chunk=256
            )
            self.assertEqual('salt', ret)

            self.assert_called_once(mock)

            args, kwargs = mock.call_args
            # expected cmd is
            # mdadm -C /dev/md0 -R -v --chunk 256 --force -l 5 -e default -n 3 /dev/sdb1 /dev/sdc1 /dev/sdd1
            # where args between -v and -l could be in any order
            self.assertEqual(len(args), 1)
            self.assertEqual(len(args[0]), 17)
            self.assertEqual(args[0][:5], [
                'mdadm',
                '-C', '/dev/md0',
                '-R',
                '-v'])
            self.assertEqual(args[0][8:], [
                 '-l', '5',
                 '-e', 'default',
                 '-n', '3',
                 '/dev/sdb1', '/dev/sdc1', '/dev/sdd1'])
            self.assertEqual(sorted(args[0][5:8]), sorted(['--chunk', '256', '--force']))
            self.assertIn('--chunk 256', ' '.join(args[0][5:8]))
            self.assertEqual(kwargs, {'python_shell': False})

    def test_create_test_mode(self):
        mock = MagicMock()
        with patch.dict(mdadm.__salt__, {'cmd.run': mock}):
            ret = mdadm.create(
                    '/dev/md0', 5,
                    devices=['/dev/sdb1', '/dev/sdc1', '/dev/sdd1'],
                    force=True,
                    chunk=256,
                    test_mode=True
            )
            self.assertEqual(sorted('mdadm -C /dev/md0 -R -v --chunk 256 '
                              '--force -l 5 -e default -n 3 '
                              '/dev/sdb1 /dev/sdc1 /dev/sdd1'.split()), sorted(ret.split()))
            assert not mock.called, 'test mode failed, cmd.run called'



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals
import os
import logging

import salt.utils.files
from salt.ext import six

from tests.support.helpers import with_tempfile
from tests.support.paths import BASE_FILES
from tests.support.case import ModuleCase
from tests.support.unit import skipIf
from tests.support.mixins import SaltReturnAssertsMixin

try:
    import M2Crypto  # pylint: disable=W0611
    HAS_M2CRYPTO = True
except ImportError:
    HAS_M2CRYPTO = False


log = logging.getLogger(__name__)


@skipIf(not HAS_M2CRYPTO, 'Skip when no M2Crypto found')
class x509Test(ModuleCase, SaltReturnAssertsMixin):

    @classmethod
    def setUpClass(cls):
        cert_path = os.path.join(BASE_FILES, 'x509_test.crt')
        with salt.utils.files.fopen(cert_path) as fp:
            cls.x509_cert_text = fp.read()

    def run_function(self, *args, **kwargs):
        ret = super(x509Test, self).run_function(*args, **kwargs)
        log.debug('ret = %s', ret)
        return ret

    @with_tempfile(suffix='.pem', create=False)
    def test_issue_49027(self, pemfile):
        ret = self.run_state(
            'x509.pem_managed',
            name=pemfile,
            text=self.x509_cert_text)
        assert isinstance(ret, dict), ret
        ret = ret[next(iter(ret))]
        assert ret.get('result') is True, ret
        with salt.utils.files.fopen(pemfile) as fp:
            result = fp.readlines()
        self.assertEqual(self.x509_cert_text.splitlines(True), result)

    @with_tempfile(suffix='.crt', create=False)
    @with_tempfile(suffix='.key', create=False)
    def test_issue_49008(self, keyfile, crtfile):
        ret = self.run_function(
            'state.apply',
            ['issue-49008'],
            pillar={'keyfile': keyfile, 'crtfile': crtfile})
        assert isinstance(ret, dict), ret
        for state_result in six.itervalues(ret):
            assert state_result['result'] is True, state_result
        assert os.path.exists(keyfile)
        assert os.path.exists(crtfile)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import MagicMock, patch

# Import Salt libs
import salt.modules.disk as disk
import salt.utils.path

STUB_DISK_USAGE = {
                   '/': {'filesystem': None, '1K-blocks': 10000, 'used': 10000, 'available': 10000, 'capacity': 10000},
                   '/dev': {'filesystem': None, '1K-blocks': 10000, 'used': 10000, 'available': 10000, 'capacity': 10000},
                   '/run': {'filesystem': None, '1K-blocks': 10000, 'used': 10000, 'available': 10000, 'capacity': 10000},
                   '/run/lock': {'filesystem': None, '1K-blocks': 10000, 'used': 10000, 'available': 10000, 'capacity': 10000},
                   '/run/shm': {'filesystem': None, '1K-blocks': 10000, 'used': 10000, 'available': 10000, 'capacity': 10000},
                   '/run/user': {'filesystem': None, '1K-blocks': 10000, 'used': 10000, 'available': 10000, 'capacity': 10000},
                   '/sys/fs/cgroup': {'filesystem': None, '1K-blocks': 10000, 'used': 10000, 'available': 10000, 'capacity': 10000}
                   }


STUB_DISK_INODEUSAGE = {
                   '/': {'inodes': 10000, 'used': 10000, 'free': 10000, 'use': 10000, 'filesystem': None},
                   '/dev': {'inodes': 10000, 'used': 10000, 'free': 10000, 'use': 10000, 'filesystem': None},
                   '/run': {'inodes': 10000, 'used': 10000, 'free': 10000, 'use': 10000, 'filesystem': None},
                   '/run/lock': {'inodes': 10000, 'used': 10000, 'free': 10000, 'use': 10000, 'filesystem': None},
                   '/run/shm': {'inodes': 10000, 'used': 10000, 'free': 10000, 'use': 10000, 'filesystem': None},
                   '/run/user': {'inodes': 10000, 'used': 10000, 'free': 10000, 'use': 10000, 'filesystem': None},
                   '/sys/fs/cgroup': {'inodes': 10000, 'used': 10000, 'free': 10000, 'use': 10000, 'filesystem': None}
                   }

STUB_DISK_PERCENT = {
                   '/': 50,
                   '/dev': 10,
                   '/run': 10,
                   '/run/lock': 10,
                   '/run/shm': 10,
                   '/run/user': 10,
                   '/sys/fs/cgroup': 10
                   }

STUB_DISK_BLKID = {'/dev/sda': {'TYPE': 'ext4', 'UUID': None}}


class DiskTestCase(TestCase, LoaderModuleMockMixin):
    '''
    TestCase for salt.modules.disk module
    '''
    def setup_loader_modules(self):
        return {disk: {}}

    def test_usage_dict(self):
        with patch.dict(disk.__grains__, {'kernel': 'Linux'}), \
                patch('salt.modules.disk.usage',
                      MagicMock(return_value=STUB_DISK_USAGE)):
            mock_cmd = MagicMock(return_value=1)
            with patch.dict(disk.__salt__, {'cmd.run': mock_cmd}):
                self.assertDictEqual(STUB_DISK_USAGE, disk.usage(args=None))

    def test_usage_none(self):
        with patch.dict(disk.__grains__, {'kernel': 'Linux'}), \
                patch('salt.modules.disk.usage', MagicMock(return_value='')):
            mock_cmd = MagicMock(return_value=1)
            with patch.dict(disk.__salt__, {'cmd.run': mock_cmd}):
                self.assertEqual('', disk.usage(args=None))

    def test_inodeusage(self):
        with patch.dict(disk.__grains__, {'kernel': 'OpenBSD'}), \
                patch('salt.modules.disk.inodeusage',
                       MagicMock(return_value=STUB_DISK_INODEUSAGE)):
            mock = MagicMock()
            with patch.dict(disk.__salt__, {'cmd.run': mock}):
                self.assertDictEqual(STUB_DISK_INODEUSAGE, disk.inodeusage(args=None))

    def test_percent(self):
        with patch.dict(disk.__grains__, {'kernel': 'Linux'}), \
                patch('salt.modules.disk.percent',
                      MagicMock(return_value=STUB_DISK_PERCENT)):
            mock = MagicMock()
            with patch.dict(disk.__salt__, {'cmd.run': mock}):
                self.assertDictEqual(STUB_DISK_PERCENT, disk.percent(args=None))

    def test_percent_args(self):
        with patch.dict(disk.__grains__, {'kernel': 'Linux'}), \
                patch('salt.modules.disk.percent', MagicMock(return_value='/')):
            mock = MagicMock()
            with patch.dict(disk.__salt__, {'cmd.run': mock}):
                self.assertEqual('/', disk.percent('/'))

    def test_blkid(self):
        with patch.dict(disk.__salt__, {'cmd.run_stdout': MagicMock(return_value=1)}), \
                patch('salt.modules.disk.blkid', MagicMock(return_value=STUB_DISK_BLKID)):
            self.assertDictEqual(STUB_DISK_BLKID, disk.blkid())

    def test_dump(self):
        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
        with patch.dict(disk.__salt__, {'cmd.run_all': mock}):
            disk.dump('/dev/sda')
            mock.assert_called_once_with(
                'blockdev --getro --getsz --getss --getpbsz --getiomin '
                '--getioopt --getalignoff --getmaxsect --getsize '
                '--getsize64 --getra --getfra /dev/sda',
                python_shell=False
            )

    def test_wipe(self):
        mock = MagicMock(return_value={'retcode': 0, 'stdout': ''})
        with patch.dict(disk.__salt__, {'cmd.run_all': mock}):
            disk.wipe('/dev/sda')
            mock.assert_called_once_with(
                'wipefs -a /dev/sda',
                python_shell=False
            )

    def test_tune(self):
        mock = MagicMock(return_value='712971264\n512\n512\n512\n0\n0\n88\n712971264\n365041287168\n512\n512')
        with patch.dict(disk.__salt__, {'cmd.run': mock}):
            mock_dump = MagicMock(return_value={'retcode': 0, 'stdout': ''})
            with patch('salt.modules.disk.dump', mock_dump):
                kwargs = {'read-ahead': 512, 'filesystem-read-ahead': 1024}
                disk.tune('/dev/sda', **kwargs)

                self.assert_called_once(mock)

                args, kwargs = mock.call_args

                # Assert called once with either 'blockdev --setra 512 --setfra 512 /dev/sda' or
                # 'blockdev --setfra 512 --setra 512 /dev/sda' and python_shell=False kwarg.
                self.assertEqual(len(args), 1)
                self.assertTrue(args[0].startswith('blockdev '))
                self.assertTrue(args[0].endswith(' /dev/sda'))
                self.assertIn(' --setra 512 ', args[0])
                self.assertIn(' --setfra 1024 ', args[0])
                self.assertEqual(len(args[0].split()), 6)
                self.assertEqual(kwargs, {'python_shell': False})

    def test_format(self):
        '''
        unit tests for disk.format
        '''
        device = '/dev/sdX1'
        mock = MagicMock(return_value=0)
        with patch.dict(disk.__salt__, {'cmd.retcode': mock}),\
               patch('salt.utils.path.which', MagicMock(return_value=True)):
            self.assertEqual(disk.format_(device), True)

    @skipIf(not salt.utils.path.which('lsblk') and not salt.utils.path.which('df'),
            'lsblk or df not found')
    def test_fstype(self):
        '''
        unit tests for disk.fstype
        '''
        device = '/dev/sdX1'
        fs_type = 'ext4'
        mock = MagicMock(return_value='FSTYPE\n{0}'.format(fs_type))
        with patch.dict(disk.__grains__, {'kernel': 'Linux'}), \
                patch.dict(disk.__salt__, {'cmd.run': mock}), \
                patch('salt.utils.path.which', MagicMock(return_value=True)):
            self.assertEqual(disk.fstype(device), fs_type)

    def test_resize2fs(self):
        '''
        unit tests for disk.resize2fs
        '''
        device = '/dev/sdX1'
        mock = MagicMock()
        with patch.dict(disk.__salt__, {'cmd.run_all': mock}), \
                patch('salt.utils.path.which', MagicMock(return_value=True)):
            disk.resize2fs(device)
            mock.assert_called_once_with('resize2fs {0}'.format(device), python_shell=False)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Alexandru Bleotu <alexandru.bleotu@morganstanley.com>`

    Tests for esxcluster proxy
'''

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import external libs
try:
    import jsonschema
    HAS_JSONSCHEMA = True
except ImportError:
    HAS_JSONSCHEMA = False

# Import Salt Libs
import salt.proxy.esxcluster as esxcluster
import salt.exceptions
from salt.config.schemas.esxcluster import EsxclusterProxySchema

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_JSONSCHEMA, 'jsonschema is required')
class InitTestCase(TestCase, LoaderModuleMockMixin):
    '''Tests for salt.proxy.esxcluster.init'''
    def setup_loader_modules(self):
        return {esxcluster: {'__virtual__':
                             MagicMock(return_value='esxcluster'),
                             'DETAILS': {}, '__pillar__': {}}}

    def setUp(self):
        self.opts_userpass = {'proxy': {'proxytype': 'esxcluster',
                                        'vcenter': 'fake_vcenter',
                                        'datacenter': 'fake_dc',
                                        'cluster': 'fake_cluster',
                                        'mechanism': 'userpass',
                                        'username': 'fake_username',
                                        'passwords': ['fake_password'],
                                        'protocol': 'fake_protocol',
                                        'port': 100}}
        self.opts_sspi = {'proxy': {'proxytype': 'esxcluster',
                                    'vcenter': 'fake_vcenter',
                                    'datacenter': 'fake_dc',
                                    'cluster': 'fake_cluster',
                                    'mechanism': 'sspi',
                                    'domain': 'fake_domain',
                                    'principal': 'fake_principal',
                                    'protocol': 'fake_protocol',
                                    'port': 100}}
        patches = (('salt.proxy.esxcluster.merge',
                    MagicMock(return_value=self.opts_sspi['proxy'])),)
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def test_merge(self):
        mock_pillar_proxy = MagicMock()
        mock_opts_proxy = MagicMock()
        mock_merge = MagicMock(return_value=self.opts_sspi['proxy'])
        with patch.dict(esxcluster.__pillar__,
                        {'proxy': mock_pillar_proxy}):
            with patch('salt.proxy.esxcluster.merge', mock_merge):
                esxcluster.init(opts={'proxy': mock_opts_proxy})
        mock_merge.assert_called_once_with(mock_opts_proxy, mock_pillar_proxy)

    def test_esxcluster_schema(self):
        mock_json_validate = MagicMock()
        serialized_schema = EsxclusterProxySchema().serialize()
        with patch('salt.proxy.esxcluster.jsonschema.validate',
                   mock_json_validate):
            esxcluster.init(self.opts_sspi)
        mock_json_validate.assert_called_once_with(
            self.opts_sspi['proxy'], serialized_schema)

    def test_invalid_proxy_input_error(self):
        with patch('salt.proxy.esxcluster.jsonschema.validate',
                   MagicMock(side_effect=jsonschema.exceptions.ValidationError(
                       'Validation Error'))):
            with self.assertRaises(salt.exceptions.InvalidConfigError) as \
                    excinfo:
                esxcluster.init(self.opts_userpass)
        self.assertEqual(excinfo.exception.strerror,
                         'Validation Error')

    def test_no_username(self):
        opts = self.opts_userpass.copy()
        del opts['proxy']['username']
        with patch('salt.proxy.esxcluster.merge',
                   MagicMock(return_value=opts['proxy'])):
            with self.assertRaises(salt.exceptions.InvalidConfigError) as \
                    excinfo:
                esxcluster.init(opts)
        self.assertEqual(excinfo.exception.strerror,
                         'Mechanism is set to \'userpass\', but no '
                         '\'username\' key found in proxy config.')

    def test_no_passwords(self):
        opts = self.opts_userpass.copy()
        del opts['proxy']['passwords']
        with patch('salt.proxy.esxcluster.merge',
                   MagicMock(return_value=opts['proxy'])):
            with self.assertRaises(salt.exceptions.InvalidConfigError) as \
                    excinfo:
                esxcluster.init(opts)
        self.assertEqual(excinfo.exception.strerror,
                         'Mechanism is set to \'userpass\', but no '
                         '\'passwords\' key found in proxy config.')

    def test_no_domain(self):
        opts = self.opts_sspi.copy()
        del opts['proxy']['domain']
        with patch('salt.proxy.esxcluster.merge',
                   MagicMock(return_value=opts['proxy'])):
            with self.assertRaises(salt.exceptions.InvalidConfigError) as \
                    excinfo:
                esxcluster.init(opts)
        self.assertEqual(excinfo.exception.strerror,
                         'Mechanism is set to \'sspi\', but no '
                         '\'domain\' key found in proxy config.')

    def test_no_principal(self):
        opts = self.opts_sspi.copy()
        del opts['proxy']['principal']
        with patch('salt.proxy.esxcluster.merge',
                   MagicMock(return_value=opts['proxy'])):
            with self.assertRaises(salt.exceptions.InvalidConfigError) as \
                    excinfo:
                esxcluster.init(opts)
        self.assertEqual(excinfo.exception.strerror,
                         'Mechanism is set to \'sspi\', but no '
                         '\'principal\' key found in proxy config.')

    def test_find_credentials(self):
        mock_find_credentials = MagicMock(return_value=('fake_username',
                                                        'fake_password'))
        with patch('salt.proxy.esxcluster.merge',
                   MagicMock(return_value=self.opts_userpass['proxy'])):
            with patch('salt.proxy.esxcluster.find_credentials',
                       mock_find_credentials):
                esxcluster.init(self.opts_userpass)
        mock_find_credentials.assert_called_once_with()

    def test_details_userpass(self):
        mock_find_credentials = MagicMock(return_value=('fake_username',
                                                        'fake_password'))
        with patch('salt.proxy.esxcluster.merge',
                   MagicMock(return_value=self.opts_userpass['proxy'])):
            with patch('salt.proxy.esxcluster.find_credentials',
                       mock_find_credentials):
                esxcluster.init(self.opts_userpass)
        self.assertDictEqual(esxcluster.DETAILS,
                             {'vcenter': 'fake_vcenter',
                              'datacenter': 'fake_dc',
                              'cluster': 'fake_cluster',
                              'mechanism': 'userpass',
                              'username': 'fake_username',
                              'password': 'fake_password',
                              'passwords': ['fake_password'],
                              'protocol': 'fake_protocol',
                              'port': 100})

    def test_details_sspi(self):
        esxcluster.init(self.opts_sspi)
        self.assertDictEqual(esxcluster.DETAILS,
                             {'vcenter': 'fake_vcenter',
                              'datacenter': 'fake_dc',
                              'cluster': 'fake_cluster',
                              'mechanism': 'sspi',
                              'domain': 'fake_domain',
                              'principal': 'fake_principal',
                              'protocol': 'fake_protocol',
                              'port': 100})



# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.case import SSHCase
from tests.support.jinja_filters import JinjaFiltersTest


class SSHJinjaFiltersTest(SSHCase, JinjaFiltersTest):
    '''
    testing Jinja filters are available via state system & salt-ssh
    '''
    pass



# -*- coding: utf-8 -*-
'''
Tests for salt.modules.zpool

:codeauthor:    Nitin Madhok <nmadhok@clemson.edu>, Jorge Schrauwen <sjorge@blackdot.be>
:maintainer:    Jorge Schrauwen <sjorge@blackdot.be>
:maturity:      new
:depends:       salt.utils.zfs
:platform:      illumos,freebsd,linux
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON,
)

# Import test data from salt.utils.zfs test
from tests.unit.utils.test_zfs import utils_patch

# Import Salt Execution module to test
import salt.utils.zfs
import salt.modules.zpool as zpool

# Import Salt Utils
import salt.loader
from salt.utils.odict import OrderedDict
import salt.utils.decorators
import salt.utils.decorators.path


# Skip this test case if we don't have access to mock!
@skipIf(NO_MOCK, NO_MOCK_REASON)
class ZpoolTestCase(TestCase, LoaderModuleMockMixin):
    '''
    This class contains a set of functions that test salt.modules.zpool module
    '''
    def setup_loader_modules(self):
        self.opts = opts = salt.config.DEFAULT_MINION_OPTS
        utils = salt.loader.utils(
            opts,
            whitelist=['zfs', 'args', 'systemd', 'path', 'platform'])
        zpool_obj = {
            zpool: {
                '__opts__': opts,
                '__utils__': utils,
            }
        }

        return zpool_obj

    def test_exists_success(self):
        '''
        Tests successful return of exists function
        '''
        ret = {}
        ret['stdout'] = "NAME      SIZE  ALLOC   FREE    CAP  DEDUP  HEALTH  ALTROOT\n" \
                        "myzpool   149G   128K   149G     0%  1.00x  ONLINE  -"
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertTrue(zpool.exists('myzpool'))

    def test_exists_failure(self):
        '''
        Tests failure return of exists function
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot open 'myzpool': no such pool"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertFalse(zpool.exists('myzpool'))

    def test_healthy(self):
        '''
        Tests successful return of healthy function
        '''
        ret = {}
        ret['stdout'] = "all pools are healthy"
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            self.assertTrue(zpool.healthy())

    def test_status(self):
        '''
        Tests successful return of status function
        '''
        ret = {}
        ret['stdout'] = "\n".join([
            "  pool: mypool",
            " state: ONLINE",
            "  scan: scrub repaired 0 in 0h6m with 0 errors on Mon Dec 21 02:06:17 2015",
            "config:",
            "",
            "\tNAME        STATE     READ WRITE CKSUM",
            "\tmypool      ONLINE       0     0     0",
            "\t  mirror-0  ONLINE       0     0     0",
            "\t    c2t0d0  ONLINE       0     0     0",
            "\t    c2t1d0  ONLINE       0     0     0",
            "",
            "errors: No known data errors",
        ])
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
                patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.status()
            self.assertEqual('ONLINE', ret['mypool']['state'])

    def test_iostat(self):
        '''
        Tests successful return of iostat function
        '''
        ret = {}
        ret['stdout'] = "\n".join([
            "               capacity     operations    bandwidth",
            "pool        alloc   free   read  write   read  write",
            "----------  -----  -----  -----  -----  -----  -----",
            "mypool      46.7G  64.3G      4     19   113K   331K",
            "  mirror    46.7G  64.3G      4     19   113K   331K",
            "    c2t0d0      -      -      1     10   114K   334K",
            "    c2t1d0      -      -      1     10   114K   334K",
            "----------  -----  -----  -----  -----  -----  -----",
        ])
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.iostat('mypool', parsable=False)
            self.assertEqual('46.7G', ret['mypool']['capacity-alloc'])

    def test_iostat_parsable(self):
        '''
        Tests successful return of iostat function

        .. note:
            The command output is the same as the non parsable!
            There is no -p flag for zpool iostat, but our type
            conversions can handle this!
        '''
        ret = {}
        ret['stdout'] = "\n".join([
            "               capacity     operations    bandwidth",
            "pool        alloc   free   read  write   read  write",
            "----------  -----  -----  -----  -----  -----  -----",
            "mypool      46.7G  64.3G      4     19   113K   331K",
            "  mirror    46.7G  64.3G      4     19   113K   331K",
            "    c2t0d0      -      -      1     10   114K   334K",
            "    c2t1d0      -      -      1     10   114K   334K",
            "----------  -----  -----  -----  -----  -----  -----",
        ])
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.iostat('mypool', parsable=True)
            self.assertEqual(50143743180, ret['mypool']['capacity-alloc'])

    def test_list(self):
        '''
        Tests successful return of list function
        '''
        ret = {}
        ret['stdout'] = "mypool\t1.81T\t661G\t1.17T\t35%\t11%\tONLINE"
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
                patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.list_(parsable=False)
            res = OrderedDict([('mypool', OrderedDict([
                ('size', '1.81T'),
                ('alloc', '661G'),
                ('free', '1.17T'),
                ('cap', '35%'),
                ('frag', '11%'),
                ('health', 'ONLINE'),
            ]))])
            self.assertEqual(ret, res)

    def test_list_parsable(self):
        '''
        Tests successful return of list function with parsable output
        '''
        ret = {}
        ret['stdout'] = "mypool\t1.81T\t661G\t1.17T\t35%\t11%\tONLINE"
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
                patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.list_(parsable=True)
            res = OrderedDict([('mypool', OrderedDict([
                ('size', 1990116046274),
                ('alloc', 709743345664),
                ('free', 1286428604497),
                ('cap', '35%'),
                ('frag', '11%'),
                ('health', 'ONLINE'),
            ]))])
            self.assertEqual(ret, res)

    def test_get(self):
        '''
        Tests successful return of get function
        '''
        ret = {}
        ret['stdout'] = "mypool\tsize\t1.81T\t-\n"
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.get('mypool', 'size', parsable=False)
            res = OrderedDict(OrderedDict([('size', '1.81T')]))
            self.assertEqual(ret, res)

    def test_get_parsable(self):
        '''
        Tests successful return of get function with parsable output
        '''
        ret = {}
        ret['stdout'] = "mypool\tsize\t1.81T\t-\n"
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.get('mypool', 'size', parsable=True)
            res = OrderedDict(OrderedDict([('size', 1990116046274)]))
            self.assertEqual(ret, res)

    def test_get_whitespace(self):
        '''
        Tests successful return of get function with a string with whitespaces
        '''
        ret = {}
        ret['stdout'] = "mypool\tcomment\tmy testing pool\t-\n"
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.get('mypool', 'comment')
            res = OrderedDict(OrderedDict([('comment', "my testing pool")]))
            self.assertEqual(ret, res)

    def test_scrub_start(self):
        '''
        Tests start of scrub
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        mock_exists = MagicMock(return_value=True)

        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.scrub('mypool')
            res = OrderedDict(OrderedDict([('scrubbing', True)]))
            self.assertEqual(ret, res)

    def test_scrub_pause(self):
        '''
        Tests pause of scrub
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        mock_exists = MagicMock(return_value=True)

        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.scrub('mypool', pause=True)
            res = OrderedDict(OrderedDict([('scrubbing', False)]))
            self.assertEqual(ret, res)

    def test_scrub_stop(self):
        '''
        Tests pauze of scrub
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)
        mock_exists = MagicMock(return_value=True)

        with patch.dict(zpool.__salt__, {'zpool.exists': mock_exists}), \
             patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.scrub('mypool', stop=True)
            res = OrderedDict(OrderedDict([('scrubbing', False)]))
            self.assertEqual(ret, res)

    def test_split_success(self):
        '''
        Tests split on success
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.split('datapool', 'backuppool')
            res = OrderedDict([('split', True)])
            self.assertEqual(ret, res)

    def test_split_exist_new(self):
        '''
        Tests split on exising new pool
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "Unable to split datapool: pool already exists"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.split('datapool', 'backuppool')
            res = OrderedDict([('split', False), ('error', 'Unable to split datapool: pool already exists')])
            self.assertEqual(ret, res)

    def test_split_missing_pool(self):
        '''
        Tests split on missing source pool
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot open 'datapool': no such pool"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.split('datapool', 'backuppool')
            res = OrderedDict([('split', False), ('error', "cannot open 'datapool': no such pool")])
            self.assertEqual(ret, res)

    def test_split_not_mirror(self):
        '''
        Tests split on source pool is not a mirror
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "Unable to split datapool: Source pool must be composed only of mirrors"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.split('datapool', 'backuppool')
            res = OrderedDict([('split', False), ('error', 'Unable to split datapool: Source pool must be composed only of mirrors')])
            self.assertEqual(ret, res)

    def test_labelclear_success(self):
        '''
        Tests labelclear on succesful label removal
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.labelclear('/dev/rdsk/c0t0d0', force=False)
            res = OrderedDict([('labelcleared', True)])
            self.assertEqual(ret, res)

    def test_labelclear_nodevice(self):
        '''
        Tests labelclear on non existing device
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "failed to open /dev/rdsk/c0t0d0: No such file or directory"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.labelclear('/dev/rdsk/c0t0d0', force=False)
            res = OrderedDict([
                ('labelcleared', False),
                ('error', 'failed to open /dev/rdsk/c0t0d0: No such file or directory'),
            ])
            self.assertEqual(ret, res)

    def test_labelclear_cleared(self):
        '''
        Tests labelclear on device with no label
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "failed to read label from /dev/rdsk/c0t0d0"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.labelclear('/dev/rdsk/c0t0d0', force=False)
            res = OrderedDict([
                ('labelcleared', False),
                ('error', 'failed to read label from /dev/rdsk/c0t0d0'),
            ])
            self.assertEqual(ret, res)

    def test_labelclear_exported(self):
        '''
        Tests labelclear on device with from exported pool
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "\n".join([
            "use '-f' to override the following error:",
            '/dev/rdsk/c0t0d0 is a member of exported pool "mypool"',
        ])
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)
        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.labelclear('/dev/rdsk/c0t0d0', force=False)
            res = OrderedDict([
                ('labelcleared', False),
                ('error', 'use \'force=True\' to override the following error:\n/dev/rdsk/c0t0d0 is a member of exported pool "mypool"'),
            ])
            self.assertEqual(ret, res)

    @skipIf(not salt.utils.path.which('mkfile'), 'Cannot find mkfile executable')
    def test_create_file_vdev_success(self):
        '''
        Tests create_file_vdev when out of space
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.create_file_vdev('64M', '/vdisks/disk0')
            res = OrderedDict([
                ('/vdisks/disk0', 'created'),
            ])
            self.assertEqual(ret, res)

    @skipIf(not salt.utils.path.which('mkfile'), 'Cannot find mkfile executable')
    def test_create_file_vdev_nospace(self):
        '''
        Tests create_file_vdev when out of space
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "/vdisks/disk0: initialized 10424320 of 67108864 bytes: No space left on device"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.create_file_vdev('64M', '/vdisks/disk0')
            res = OrderedDict([
                ('/vdisks/disk0', 'failed'),
                ('error', OrderedDict([
                    ('/vdisks/disk0', ' initialized 10424320 of 67108864 bytes: No space left on device'),
                ])),
            ])
            self.assertEqual(ret, res)

    def test_export_success(self):
        '''
        Tests export
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.export('mypool')
            res = OrderedDict([('exported', True)])
            self.assertEqual(ret, res)

    def test_export_nopool(self):
        '''
        Tests export when the pool does not exists
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot open 'mypool': no such pool"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.export('mypool')
            res = OrderedDict([('exported', False), ('error', "cannot open 'mypool': no such pool")])
            self.assertEqual(ret, res)

    def test_import_success(self):
        '''
        Tests import
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.import_('mypool')
            res = OrderedDict([('imported', True)])
            self.assertEqual(ret, res)

    def test_import_duplicate(self):
        '''
        Tests import with already imported pool
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "\n".join([
            "cannot import 'mypool': a pool with that name already exists",
            "use the form 'zpool import <pool | id> <newpool>' to give it a new name",
        ])
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.import_('mypool')
            res = OrderedDict([
                ('imported', False),
                ('error', "cannot import 'mypool': a pool with that name already exists\nuse the form 'zpool import <pool | id> <newpool>' to give it a new name"),
            ])
            self.assertEqual(ret, res)

    def test_import_nopool(self):
        '''
        Tests import
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot import 'mypool': no such pool available"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.import_('mypool')
            res = OrderedDict([
                ('imported', False),
                ('error', "cannot import 'mypool': no such pool available"),
            ])
            self.assertEqual(ret, res)

    def test_online_success(self):
        '''
        Tests online
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.online('mypool', '/dev/rdsk/c0t0d0')
            res = OrderedDict([('onlined', True)])
            self.assertEqual(ret, res)

    def test_online_nodevice(self):
        '''
        Tests online
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot online /dev/rdsk/c0t0d1: no such device in pool"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.online('mypool', '/dev/rdsk/c0t0d1')
            res = OrderedDict([
                ('onlined', False),
                ('error', 'cannot online /dev/rdsk/c0t0d1: no such device in pool'),
            ])
            self.assertEqual(ret, res)

    def test_offline_success(self):
        '''
        Tests offline
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.offline('mypool', '/dev/rdsk/c0t0d0')
            res = OrderedDict([('offlined', True)])
            self.assertEqual(ret, res)

    def test_offline_nodevice(self):
        '''
        Tests offline
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot offline /dev/rdsk/c0t0d1: no such device in pool"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.offline('mypool', '/dev/rdsk/c0t0d1')
            res = OrderedDict([
                ('offlined', False),
                ('error', 'cannot offline /dev/rdsk/c0t0d1: no such device in pool'),
            ])
            self.assertEqual(ret, res)

    def test_offline_noreplica(self):
        '''
        Tests offline
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot offline /dev/rdsk/c0t0d1: no valid replicas"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.offline('mypool', '/dev/rdsk/c0t0d1')
            res = OrderedDict([
                ('offlined', False),
                ('error', 'cannot offline /dev/rdsk/c0t0d1: no valid replicas'),
            ])
            self.assertEqual(ret, res)

    def test_reguid_success(self):
        '''
        Tests reguid
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.reguid('mypool')
            res = OrderedDict([('reguided', True)])
            self.assertEqual(ret, res)

    def test_reguid_nopool(self):
        '''
        Tests reguid with missing pool
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot open 'mypool': no such pool"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.reguid('mypool')
            res = OrderedDict([
                ('reguided', False),
                ('error', "cannot open 'mypool': no such pool"),
            ])
            self.assertEqual(ret, res)

    def test_reopen_success(self):
        '''
        Tests reopen
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.reopen('mypool')
            res = OrderedDict([('reopened', True)])
            self.assertEqual(ret, res)

    def test_reopen_nopool(self):
        '''
        Tests reopen with missing pool
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot open 'mypool': no such pool"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.reopen('mypool')
            res = OrderedDict([
                ('reopened', False),
                ('error', "cannot open 'mypool': no such pool"),
            ])
            self.assertEqual(ret, res)

    def test_upgrade_success(self):
        '''
        Tests upgrade
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.upgrade('mypool')
            res = OrderedDict([('upgraded', True)])
            self.assertEqual(ret, res)

    def test_upgrade_nopool(self):
        '''
        Tests upgrade with missing pool
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot open 'mypool': no such pool"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.upgrade('mypool')
            res = OrderedDict([
                ('upgraded', False),
                ('error', "cannot open 'mypool': no such pool"),
            ])
            self.assertEqual(ret, res)

    def test_history_success(self):
        '''
        Tests history
        '''
        ret = {}
        ret['stdout'] = "\n".join([
            "History for 'mypool':",
            "2018-01-18.16:56:12 zpool create -f mypool /dev/rdsk/c0t0d0",
            "2018-01-19.16:01:55 zpool attach -f mypool /dev/rdsk/c0t0d0 /dev/rdsk/c0t0d1",
        ])
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.history('mypool')
            res = OrderedDict([
                ('mypool', OrderedDict([
                    ('2018-01-18.16:56:12', 'zpool create -f mypool /dev/rdsk/c0t0d0'),
                    ('2018-01-19.16:01:55', 'zpool attach -f mypool /dev/rdsk/c0t0d0 /dev/rdsk/c0t0d1'),
                ])),
            ])
            self.assertEqual(ret, res)

    def test_history_nopool(self):
        '''
        Tests history with missing pool
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot open 'mypool': no such pool"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.history('mypool')
            res = OrderedDict([
                ('error', "cannot open 'mypool': no such pool"),
            ])
            self.assertEqual(ret, res)

    def test_clear_success(self):
        '''
        Tests clear
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = ""
        ret['retcode'] = 0
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.clear('mypool')
            res = OrderedDict([('cleared', True)])
            self.assertEqual(ret, res)

    def test_clear_nopool(self):
        '''
        Tests clear with missing pool
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot open 'mypool': no such pool"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.clear('mypool')
            res = OrderedDict([
                ('cleared', False),
                ('error', "cannot open 'mypool': no such pool"),
            ])

    def test_clear_nodevice(self):
        '''
        Tests clear with non existign device
        '''
        ret = {}
        ret['stdout'] = ""
        ret['stderr'] = "cannot clear errors for /dev/rdsk/c0t0d0: no such device in pool"
        ret['retcode'] = 1
        mock_cmd = MagicMock(return_value=ret)

        with patch.dict(zpool.__salt__, {'cmd.run_all': mock_cmd}), \
             patch.dict(zpool.__utils__, utils_patch):
            ret = zpool.clear('mypool', '/dev/rdsk/c0t0d0')
            res = OrderedDict([
                ('cleared', False),
                ('error', "cannot clear errors for /dev/rdsk/c0t0d0: no such device in pool"),
            ])
            self.assertEqual(ret, res)
